@@ -1,6 +1,6 @@
 const t = require('tap')
-const { readdirSync } = require('fs')
-const { dirname } = require('path')
+const { readdirSync } = require('node:fs')
+const { dirname } = require('node:path')
 const { load: loadMockNpm } = require('../../fixtures/mock-npm.js')
 const tmock = require('../../fixtures/tmock.js')
 const validateEngines = require('../../../lib/cli/validate-engines.js')
@@ -8,16 +8,22 @@ const validateEngines = require('../../../lib/cli/validate-engines.js')
 const cliMock = async (t, opts) => {
   let exitHandlerArgs = null
   let npm = null
-  const exitHandlerMock = (...args) => {
-    exitHandlerArgs = args
-    npm.unload()
-  }
-  exitHandlerMock.setNpm = _npm => npm = _npm
 
   const { Npm, ...mock } = await loadMockNpm(t, { ...opts, init: false })
   const cli = tmock(t, '{LIB}/cli/entry.js', {
     '{LIB}/npm.js': Npm,
-    '{LIB}/cli/exit-handler.js': exitHandlerMock,
+    '{LIB}/cli/exit-handler.js': class MockExitHandler {
+      exit (...args) {
+        exitHandlerArgs = args
+        npm.unload()
+      }
+
+      registerUncaughtHandlers () {}
+
+      setNpm (_npm) {
+        npm = _npm
+      }
+    },
   })
 
   return {
@@ -29,16 +35,16 @@ const cliMock = async (t, opts) => {
   }
 }
 
-t.test('print the version, and treat npm_g as npm -g', async t => {
+t.test('print the version ', async t => {
   const { logs, cli, Npm, outputs, exitHandlerCalled } = await cliMock(t, {
-    globals: { 'process.argv': ['node', 'npm_g', 'root'] },
+    globals: { 'process.argv': ['node', 'npm', 'root'] },
   })
   await cli(process)
 
-  t.strictSame(process.argv, ['node', 'npm', '-g', 'root'], 'system process.argv was rewritten')
+  t.strictSame(process.argv, ['node', 'npm', 'root'], 'system process.argv was rewritten')
   t.strictSame(logs.verbose.byTitle('cli'), ['cli node npm'])
   t.strictSame(logs.verbose.byTitle('title'), ['title npm root'])
-  t.match(logs.verbose.byTitle('argv'), ['argv "--global" "root"'])
+  t.match(logs.verbose.byTitle('argv'), ['argv "root"'])
   t.strictSame(logs.info, [`using npm@${Npm.version}`, `using node@${process.version}`])
   t.equal(outputs.length, 1)
   t.match(outputs[0], dirname(process.cwd()))
@@ -113,21 +119,6 @@ t.test('print usage if no params provided', async t => {
   t.match(process.exitCode, 1)
 })
 
-t.test('print usage if non-command param provided', async t => {
-  const { cli, outputs, exitHandlerCalled, exitHandlerNpm } = await cliMock(t, {
-    globals: {
-      'process.argv': ['node', 'npm', 'tset'],
-    },
-  })
-  await cli(process)
-
-  t.match(outputs[0], 'Unknown command: "tset"')
-  t.match(outputs[0], 'Did you mean this?')
-  t.match(exitHandlerCalled(), [], 'should call exitHandler with no args')
-  t.ok(exitHandlerNpm(), 'exitHandler npm is set')
-  t.match(process.exitCode, 1)
-})
-
 t.test('load error calls error handler', async t => {
   const err = new Error('test load error')
   const { cli, exitHandlerCalled } = await cliMock(t, {
