@@ -18,6 +18,10 @@
 #include "src/objects/oddball.h"
 #include "src/runtime/runtime.h"
 
+#if DEBUG && V8_ENABLE_WEBASSEMBLY
+#include "src/wasm/canonical-types.h"
+#endif
+
 namespace v8 {
 namespace internal {
 
@@ -841,7 +845,7 @@ const ForInParameters& ForInParametersOf(const Operator* op);
 class JSWasmCallParameters {
  public:
   explicit JSWasmCallParameters(const wasm::WasmModule* module,
-                                const wasm::FunctionSig* signature,
+                                const wasm::CanonicalSig* signature,
                                 int function_index,
                                 SharedFunctionInfoRef shared_fct_info,
                                 wasm::NativeModule* native_module,
@@ -853,11 +857,11 @@ class JSWasmCallParameters {
         native_module_(native_module),
         feedback_(feedback) {
     DCHECK_NOT_NULL(module);
-    DCHECK_NOT_NULL(signature);
+    DCHECK(wasm::GetTypeCanonicalizer()->Contains(signature));
   }
 
   const wasm::WasmModule* module() const { return module_; }
-  const wasm::FunctionSig* signature() const { return signature_; }
+  const wasm::CanonicalSig* signature() const { return signature_; }
   int function_index() const { return function_index_; }
   SharedFunctionInfoRef shared_fct_info() const { return shared_fct_info_; }
   wasm::NativeModule* native_module() const { return native_module_; }
@@ -867,7 +871,7 @@ class JSWasmCallParameters {
 
  private:
   const wasm::WasmModule* const module_;
-  const wasm::FunctionSig* const signature_;
+  const wasm::CanonicalSig* const signature_;
   int function_index_;
   SharedFunctionInfoRef shared_fct_info_;
   wasm::NativeModule* native_module_;
@@ -955,6 +959,7 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
   const Operator* CreateStringIterator();
   const Operator* CreateKeyValueArray();
   const Operator* CreateObject();
+  const Operator* CreateStringWrapper();
   const Operator* CreatePromise();
   const Operator* CreateTypedArray();
   const Operator* CreateLiteralArray(ArrayBoilerplateDescriptionRef constant,
@@ -1005,7 +1010,7 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
 
 #if V8_ENABLE_WEBASSEMBLY
   const Operator* CallWasm(const wasm::WasmModule* wasm_module,
-                           const wasm::FunctionSig* wasm_signature,
+                           const wasm::CanonicalSig* wasm_signature,
                            int wasm_function_index,
                            SharedFunctionInfoRef shared_fct_info,
                            wasm::NativeModule* native_module,
@@ -1060,6 +1065,7 @@ class V8_EXPORT_PRIVATE JSOperatorBuilder final
 
   const Operator* HasContextExtension(size_t depth);
   const Operator* LoadContext(size_t depth, size_t index, bool immutable);
+  const Operator* LoadScriptContext(size_t depth, size_t index);
   const Operator* StoreContext(size_t depth, size_t index);
   const Operator* StoreScriptContext(size_t depth, size_t index);
 
@@ -1519,7 +1525,7 @@ class JSWasmCallNode final : public JSCallOrConstructNode {
     return Parameters().arity_without_implicit_args();
   }
 
-  static Type TypeForWasmReturnType(const wasm::ValueType& type);
+  static Type TypeForWasmReturnType(wasm::CanonicalValueType type);
 };
 #endif  // V8_ENABLE_WEBASSEMBLY
 
