@@ -70,7 +70,7 @@ class MaglevEarlyLoweringReducer : public Next {
   }
 
   V<InternalizedString> CheckedInternalizedString(
-      V<Object> object, OpIndex frame_state, bool check_smi,
+      V<Object> object, V<FrameState> frame_state, bool check_smi,
       const FeedbackSource& feedback) {
     if (check_smi) {
       __ DeoptimizeIf(__ IsSmi(object), frame_state, DeoptimizeReason::kSmi,
@@ -121,6 +121,118 @@ class MaglevEarlyLoweringReducer : public Next {
     }
   }
 
+  V<Object> LoadScriptContextSideData(V<Context> script_context, int index) {
+    V<FixedArray> side_table = __ template LoadTaggedField<FixedArray>(
+        script_context,
+        Context::OffsetOfElementAt(Context::CONTEXT_SIDE_TABLE_PROPERTY_INDEX));
+    return __ LoadTaggedField(side_table,
+                              FixedArray::OffsetOfElementAt(
+                                  index - Context::MIN_CONTEXT_EXTENDED_SLOTS));
+  }
+
+  V<Object> LoadScriptContextPropertyFromSideData(V<Object> side_data) {
+    ScopedVar<Object> property(this, side_data);
+    IF_NOT (__ IsSmi(side_data)) {
+      property = __ LoadTaggedField(
+          side_data, ContextSidePropertyCell::kPropertyDetailsRawOffset);
+    }
+    return property;
+  }
+
+  V<Object> LoadHeapNumberFromScriptContext(V<Context> script_context,
+                                            int index,
+                                            V<HeapNumber> heap_number) {
+    V<Object> data = __ LoadScriptContextSideData(script_context, index);
+    V<Object> property = __ LoadScriptContextPropertyFromSideData(data);
+    ScopedVar<HeapNumber> result(this, heap_number);
+    Label<> done(this);
+    if (v8_flags.script_context_mutable_heap_int32) {
+      IF (__ TaggedEqual(
+              property,
+              __ SmiConstant(ContextSidePropertyCell::MutableInt32()))) {
+        result = __ AllocateHeapNumberWithValue(
+            __ ChangeInt32ToFloat64(__ LoadHeapInt32Value(heap_number)),
+            isolate_->factory());
+        GOTO(done);
+      }
+    }
+    IF (__ TaggedEqual(
+            property,
+            __ SmiConstant(ContextSidePropertyCell::MutableHeapNumber()))) {
+      result = __ AllocateHeapNumberWithValue(
+          __ LoadHeapNumberValue(heap_number), isolate_->factory());
+    }
+    GOTO(done);
+    BIND(done);
+    return result;
+  }
+
+  void StoreScriptContextSlowPath(V<Context> script_context,
+                                  V<Object> old_value, V<Object> new_value,
+                                  V<Object> side_data,
+                                  V<FrameState> frame_state,
+                                  const FeedbackSource& feedback,
+                                  Label<>& done) {
+    // Check if Undefined.
+    __ DeoptimizeIf(
+        __ RootEqual(side_data, RootIndex::kUndefinedValue, isolate_),
+        frame_state, DeoptimizeReason::kWrongValue, feedback);
+    V<Object> property = __ LoadScriptContextPropertyFromSideData(side_data);
+    // Check for const case.
+    __ DeoptimizeIf(
+        __ TaggedEqual(property,
+                       __ SmiConstant(ContextSidePropertyCell::Const())),
+        frame_state, DeoptimizeReason::kWrongValue, feedback);
+    if (v8_flags.script_context_mutable_heap_number) {
+      // Check for smi case.
+      IF (__ TaggedEqual(
+              property, __ SmiConstant(ContextSidePropertyCell::SmiMarker()))) {
+        __ DeoptimizeIfNot(__ IsSmi(new_value), frame_state,
+                           DeoptimizeReason::kWrongValue, feedback);
+      } ELSE {
+        if (v8_flags.script_context_mutable_heap_int32) {
+          // Check for mutable heap int32 case.
+          IF (__ TaggedEqual(
+                  property,
+                  __ SmiConstant(ContextSidePropertyCell::MutableInt32()))) {
+            ScopedVar<Word32> number_value(this);
+            IF (__ IsSmi(new_value)) {
+              number_value = __ UntagSmi(V<Smi>::Cast(new_value));
+            } ELSE {
+              V<i::Map> map = __ LoadMapField(new_value);
+              __ DeoptimizeIfNot(
+                  __ TaggedEqual(map,
+                                 __ HeapConstant(factory_->heap_number_map())),
+                  frame_state, DeoptimizeReason::kWrongValue, feedback);
+              number_value = __ ChangeFloat64ToInt32OrDeopt(
+                  __ LoadHeapNumberValue(V<HeapNumber>::Cast(new_value)),
+                  frame_state, CheckForMinusZeroMode::kCheckForMinusZero,
+                  feedback);
+            }
+            __ StoreField(old_value, AccessBuilder::ForHeapInt32Value(),
+                          number_value);
+            GOTO(done);
+          }
+        }
+        // It must be a mutable heap number case.
+        ScopedVar<Float64> number_value(this);
+        IF (__ IsSmi(new_value)) {
+          number_value =
+              __ ChangeInt32ToFloat64(__ UntagSmi(V<Smi>::Cast(new_value)));
+        } ELSE {
+          V<i::Map> map = __ LoadMapField(new_value);
+          __ DeoptimizeIfNot(
+              __ TaggedEqual(map, __ HeapConstant(factory_->heap_number_map())),
+              frame_state, DeoptimizeReason::kWrongValue, feedback);
+          number_value = __ LoadHeapNumberValue(V<HeapNumber>::Cast(new_value));
+        }
+        __ StoreField(old_value, AccessBuilder::ForHeapNumberValue(),
+                      number_value);
+        GOTO(done);
+      }
+    }
+  }
+
   V<Object> CheckConstructResult(V<Object> construct_result,
                                  V<Object> implicit_receiver) {
     // If the result is an object (in the ECMA sense), we should get rid
@@ -154,49 +266,25 @@ class MaglevEarlyLoweringReducer : public Next {
     // The result of a derived construct should be an object (in the ECMA
     // sense).
     Label<> do_throw(this);
+    Label<> end(this);
 
     // If the result is a smi, it is *not* an object in the ECMA sense.
-    GOTO_IF(__ IsSmi(construct_result), do_throw);
+    GOTO_IF(UNLIKELY(__ IsSmi(construct_result)), do_throw);
 
-    // Check if the type of the result is not an object done the ECMA sense.
-    IF_NOT (JSAnyIsNotPrimitive(V<HeapObject>::Cast(construct_result))) {
-      GOTO(do_throw);
-      BIND(do_throw);
+    // Check if the type of the result is not an object in the ECMA sense.
+    GOTO_IF(LIKELY(JSAnyIsNotPrimitive(V<HeapObject>::Cast(construct_result))),
+            end);
+    GOTO(do_throw);
+
+    BIND(do_throw);
+    {
       __ CallRuntime_ThrowConstructorReturnedNonObject(
           isolate_, frame_state, native_context, lazy_deopt_on_throw);
       // ThrowConstructorReturnedNonObject should not return.
       __ Unreachable();
     }
-  }
 
-  void CheckConstTrackingLetCellTagged(V<Context> context, V<Object> value,
-                                       int index, V<FrameState> frame_state,
-                                       const FeedbackSource& feedback) {
-    V<Object> old_value =
-        __ LoadTaggedField(context, Context::OffsetOfElementAt(index));
-    IF_NOT (__ TaggedEqual(old_value, value)) {
-      CheckConstTrackingLetCell(context, index, frame_state, feedback);
-    }
-  }
-
-  void CheckConstTrackingLetCell(V<Context> context, int index,
-                                 V<FrameState> frame_state,
-                                 const FeedbackSource& feedback) {
-    // Load the const tracking let side data.
-    V<Object> side_data = __ LoadTaggedField(
-        context, Context::OffsetOfElementAt(
-                     Context::CONST_TRACKING_LET_SIDE_DATA_INDEX));
-    V<Object> index_data = __ LoadTaggedField(
-        side_data, FixedArray::OffsetOfElementAt(
-                       index - Context::MIN_CONTEXT_EXTENDED_SLOTS));
-    // If the field is already marked as "not a constant", storing a
-    // different value is fine. But if it's anything else (including the hole,
-    // which means no value was stored yet), deopt this code. The lower tier
-    // code will update the side data and invalidate DependentCode if needed.
-    V<Word32> is_const = __ TaggedEqual(
-        index_data, __ SmiConstant(ConstTrackingLetCell::kNonConstMarker));
-    __ DeoptimizeIfNot(is_const, frame_state,
-                       DeoptimizeReason::kConstTrackingLet, feedback);
+    BIND(end);
   }
 
   V<Smi> UpdateJSArrayLength(V<Word32> length_raw, V<JSArray> object,
@@ -218,57 +306,27 @@ class MaglevEarlyLoweringReducer : public Next {
     return length_tagged;
   }
 
-  void TransitionElementsKindOrCheckMap(
-      V<Object> object, V<FrameState> frame_state, bool check_heap_object,
+  V<Map> TransitionMultipleElementsKind(
+      V<Object> object, V<Map> map,
       const ZoneVector<compiler::MapRef>& transition_sources,
-      const MapRef transition_target, const FeedbackSource& feedback) {
-    Label<> end(this);
-    Label<> if_smi(this);
-
-    TransitionElementsKind(object, transition_sources, transition_target,
-                           check_heap_object, if_smi, end);
-
-    __ DeoptimizeIfNot(
-        __ TaggedEqual(__ LoadMapField(object),
-                       __ HeapConstant(transition_target.object())),
-        frame_state, DeoptimizeReason::kWrongMap, feedback);
-    GOTO(end);
-
-    if (check_heap_object && if_smi.has_incoming_jump()) {
-      BIND(if_smi);
-      __ Deoptimize(frame_state, DeoptimizeReason::kSmi, feedback);
-    } else {
-      DCHECK(!if_smi.has_incoming_jump());
-    }
-
-    BIND(end);
-  }
-
-  void TransitionMultipleElementsKind(
-      V<Object> object, const ZoneVector<compiler::MapRef>& transition_sources,
       const MapRef transition_target) {
-    Label<> end(this);
+    Label<Map> end(this);
 
-    TransitionElementsKind(object, transition_sources, transition_target,
-                           /* check_heap_object */ true, end, end);
-
-    GOTO(end);
-    BIND(end);
+    TransitionElementsKind(object, map, transition_sources, transition_target,
+                           end);
+    GOTO(end, map);
+    BIND(end, result);
+    return result;
   }
 
   void TransitionElementsKind(
-      V<Object> object, const ZoneVector<compiler::MapRef>& transition_sources,
-      const MapRef transition_target, bool check_heap_object, Label<>& if_smi,
-      Label<>& end) {
-    if (check_heap_object) {
-      GOTO_IF(__ ObjectIsSmi(object), if_smi);
-    }
-
+      V<Object> object, V<Map> map,
+      const ZoneVector<compiler::MapRef>& transition_sources,
+      const MapRef transition_target, Label<Map>& end) {
     // Turboshaft's TransitionElementsKind operation loads the map everytime, so
     // we don't call it to have a single map load (in practice,
     // LateLoadElimination should probably eliminate the subsequent map loads,
     // but let's not risk it).
-    V<Map> map = __ LoadMapField(object);
     V<Map> target_map = __ HeapConstant(transition_target.object());
 
     for (const compiler::MapRef transition_source : transition_sources) {
@@ -282,14 +340,14 @@ class MaglevEarlyLoweringReducer : public Next {
               isolate_, __ NoContextConstant(), V<HeapObject>::Cast(object),
               target_map);
         }
-        GOTO(end);
+        GOTO(end, target_map);
       }
     }
   }
 
   V<Word32> JSAnyIsNotPrimitive(V<HeapObject> heap_object) {
     V<Map> map = __ LoadMapField(heap_object);
-    if (V8_STATIC_ROOTS_BOOL) {
+    if constexpr (V8_STATIC_ROOTS_BOOL) {
       // All primitive object's maps are allocated at the start of the read only
       // heap. Thus JS_RECEIVER's must have maps with larger (compressed)
       // addresses.
@@ -365,12 +423,12 @@ class MaglevEarlyLoweringReducer : public Next {
         __ template LoadField<Word32>(map, AccessBuilder::ForMapBitField3());
     IF (UNLIKELY(__ Word32BitwiseAnd(bitfield3,
                                      Map::Bits3::IsDeprecatedBit::kMask))) {
-      V<Object> result = __ CallRuntime_TryMigrateInstance(
+      V<Object> object_or_smi = __ CallRuntime_TryMigrateInstance(
           isolate_, __ NoContextConstant(), object);
-      __ DeoptimizeIf(__ ObjectIsSmi(result), frame_state,
+      __ DeoptimizeIf(__ ObjectIsSmi(object_or_smi), frame_state,
                       DeoptimizeReason::kInstanceMigrationFailed, feedback);
       // Reload the map since TryMigrateInstance might have changed it.
-      result = __ LoadMapField(V<HeapObject>::Cast(result));
+      result = __ LoadMapField(V<HeapObject>::Cast(object_or_smi));
     }
 
     return result;
