@@ -31,10 +31,10 @@ class SourceCodeCache final {
   void Iterate(RootVisitor* v);
 
   bool Lookup(Isolate* isolate, base::Vector<const char> name,
-              Handle<SharedFunctionInfo>* handle);
+              DirectHandle<SharedFunctionInfo>* handle);
 
   void Add(Isolate* isolate, base::Vector<const char> name,
-           Handle<SharedFunctionInfo> shared);
+           DirectHandle<SharedFunctionInfo> shared);
 
  private:
   Script::Type type_;
@@ -56,8 +56,8 @@ class Bootstrapper final {
 
   // Creates a JavaScript Global Context with initial object graph.
   // The returned value is a global handle casted to V8Environment*.
-  Handle<NativeContext> CreateEnvironment(
-      MaybeHandle<JSGlobalProxy> maybe_global_proxy,
+  DirectHandle<NativeContext> CreateEnvironment(
+      MaybeDirectHandle<JSGlobalProxy> maybe_global_proxy,
       v8::Local<v8::ObjectTemplate> global_object_template,
       v8::ExtensionConfiguration* extensions, size_t context_snapshot_index,
       DeserializeEmbedderFieldsCallback embedder_fields_deserializer,
@@ -65,8 +65,8 @@ class Bootstrapper final {
 
   // Used for testing context deserialization. No code runs in the generated
   // context. It only needs to pass heap verification.
-  Handle<NativeContext> CreateEnvironmentForTesting() {
-    MaybeHandle<JSGlobalProxy> no_global_proxy;
+  DirectHandle<NativeContext> CreateEnvironmentForTesting() {
+    MaybeDirectHandle<JSGlobalProxy> no_global_proxy;
     v8::Local<v8::ObjectTemplate> no_global_object_template;
     ExtensionConfiguration no_extensions;
     static constexpr int kDefaultContextIndex = 0;
@@ -77,8 +77,8 @@ class Bootstrapper final {
                              no_microtask_queue);
   }
 
-  Handle<JSGlobalProxy> NewRemoteContext(
-      MaybeHandle<JSGlobalProxy> maybe_global_proxy,
+  DirectHandle<JSGlobalProxy> NewRemoteContext(
+      MaybeDirectHandle<JSGlobalProxy> maybe_global_proxy,
       v8::Local<v8::ObjectTemplate> global_object_template);
 
   // Traverses the pointers for memory management.
@@ -94,7 +94,7 @@ class Bootstrapper final {
   void FreeThreadResources();
 
   // Used for new context creation.
-  bool InstallExtensions(Handle<NativeContext> native_context,
+  bool InstallExtensions(DirectHandle<NativeContext> native_context,
                          v8::ExtensionConfiguration* extensions);
 
   SourceCodeCache* extensions_cache() { return &extensions_cache_; }
@@ -130,9 +130,17 @@ class BootstrapperActive final {
   Bootstrapper* bootstrapper_;
 };
 
+// Exposed for Wasm bootstrapping.
 V8_NOINLINE Handle<JSFunction> SimpleInstallFunction(
-    Isolate* isolate, Handle<JSObject> base, const char* name, Builtin call,
-    int len, bool adapt, PropertyAttributes attrs = DONT_ENUM);
+    Isolate* isolate, DirectHandle<JSObject> base, const char* name,
+    Builtin call, int len, AdaptArguments adapt,
+    PropertyAttributes attrs = DONT_ENUM);
+
+// Exposed for Wasm bootstrapping.
+V8_NOINLINE void InstallError(
+    Isolate* isolate, DirectHandle<JSObject> global, DirectHandle<String> name,
+    int context_index, Builtin error_constructor = Builtin::kErrorConstructor,
+    int error_function_length = 1);
 
 }  // namespace internal
 }  // namespace v8
