@@ -22,8 +22,6 @@ class AllocationSite : public Struct {
  public:
   NEVER_READ_ONLY_SPACE
   static const uint32_t kMaximumArrayBytesToPretransition = 8 * 1024;
-  static const double kPretenureRatio;
-  static const int kPretenureMinimumCreated = 100;
 
   // Values for pretenure decision field.
   enum PretenureDecision {
@@ -39,27 +37,27 @@ class AllocationSite : public Struct {
 
   // Contains either a Smi-encoded bitfield or a boilerplate. If it's a Smi the
   // AllocationSite is for a constructed Array.
-  DECL_ACCESSORS(transition_info_or_boilerplate, Object)
-  DECL_RELEASE_ACQUIRE_ACCESSORS(transition_info_or_boilerplate, Object)
-  DECL_GETTER(boilerplate, JSObject)
-  DECL_RELEASE_ACQUIRE_ACCESSORS(boilerplate, JSObject)
+  DECL_ACCESSORS(transition_info_or_boilerplate, Tagged<Object>)
+  DECL_RELEASE_ACQUIRE_ACCESSORS(transition_info_or_boilerplate, Tagged<Object>)
+  DECL_GETTER(boilerplate, Tagged<JSObject>)
+  DECL_RELEASE_ACQUIRE_ACCESSORS(boilerplate, Tagged<JSObject>)
   DECL_INT_ACCESSORS(transition_info)
 
   // nested_site threads a list of sites that represent nested literals
   // walked in a particular order. So [[1, 2], 1, 2] will have one
   // nested_site, but [[1, 2], 3, [4]] will have a list of two.
-  DECL_ACCESSORS(nested_site, Object)
+  DECL_ACCESSORS(nested_site, Tagged<Object>)
 
   // Bitfield containing pretenuring information.
   DECL_RELAXED_INT32_ACCESSORS(pretenure_data)
 
   DECL_INT32_ACCESSORS(pretenure_create_count)
-  DECL_ACCESSORS(dependent_code, DependentCode)
+  DECL_ACCESSORS(dependent_code, Tagged<DependentCode>)
 
   // heap->allocation_site_list() points to the last AllocationSite which form
   // a linked list through the weak_next property. The GC might remove elements
   // from the list by updateing weak_next.
-  DECL_ACCESSORS(weak_next, Object)
+  DECL_ACCESSORS(weak_next, Tagged<Object>)
 
   inline void Initialize();
 
@@ -78,11 +76,10 @@ class AllocationSite : public Struct {
   using MementoFoundCountBits = base::BitField<int, 0, 26>;
   using PretenureDecisionBits = base::BitField<PretenureDecision, 26, 3>;
   using DeoptDependentCodeBit = base::BitField<bool, 29, 1>;
-  STATIC_ASSERT(PretenureDecisionBits::kMax >= kLastPretenureDecisionValue);
+  static_assert(PretenureDecisionBits::kMax >= kLastPretenureDecisionValue);
 
-  // Increments the mementos found counter and returns true when the first
-  // memento was found for a given allocation site.
-  inline bool IncrementMementoFoundCount(int increment = 1);
+  // Increments the mementos found counter and returns the new count.
+  inline int IncrementMementoFoundCount(int increment = 1);
 
   inline void IncrementMementoCreateCount();
 
@@ -107,9 +104,11 @@ class AllocationSite : public Struct {
   // it may be that in new space there are AllocationMementos hanging around
   // which point to the AllocationSite. If we scavenge these AllocationSites
   // too soon, those AllocationMementos will end up pointing to garbage
-  // addresses. The garbage collector marks such AllocationSites as zombies
-  // when it discovers there are no roots, allowing the subsequent collection
-  // pass to recognize zombies and discard them later.
+  // addresses. The concrete case happens when evacuating new space in the full
+  // GC which happens after sweeping has been started already. To mitigate this
+  // problem the garbage collector marks such AllocationSites as zombies when it
+  // discovers there are no roots, allowing the subsequent collection pass to
+  // recognize zombies and discard them later.
   inline bool IsZombie() const;
 
   inline bool IsMaybeTenure() const;
@@ -131,13 +130,12 @@ class AllocationSite : public Struct {
 
   template <AllocationSiteUpdateMode update_or_check =
                 AllocationSiteUpdateMode::kUpdate>
-  static bool DigestTransitionFeedback(Handle<AllocationSite> site,
+  static bool DigestTransitionFeedback(DirectHandle<AllocationSite> site,
                                        ElementsKind to_kind);
 
   DECL_PRINTER(AllocationSite)
   DECL_VERIFIER(AllocationSite)
 
-  DECL_CAST(AllocationSite)
   static inline bool ShouldTrack(ElementsKind boilerplate_elements_kind);
   static bool ShouldTrack(ElementsKind from, ElementsKind to);
   static inline bool CanTrack(InstanceType type);
@@ -173,14 +171,16 @@ class AllocationSite : public Struct {
 class AllocationMemento
     : public TorqueGeneratedAllocationMemento<AllocationMemento, Struct> {
  public:
-  DECL_ACCESSORS(allocation_site, Object)
+  DECL_ACCESSORS(allocation_site, Tagged<Object>)
 
   inline bool IsValid() const;
-  inline AllocationSite GetAllocationSite() const;
+  inline Tagged<AllocationSite> GetAllocationSite() const;
   inline Address GetAllocationSiteUnchecked() const;
 
   DECL_PRINTER(AllocationMemento)
 
+  using BodyDescriptor = StructBodyDescriptor;
+
   TQ_OBJECT_CONSTRUCTORS(AllocationMemento)
 };
 
