@@ -4,17 +4,20 @@
 
 #include "src/regexp/experimental/experimental.h"
 
+#include <optional>
+
 #include "src/common/assert-scope.h"
 #include "src/objects/js-regexp-inl.h"
 #include "src/regexp/experimental/experimental-compiler.h"
 #include "src/regexp/experimental/experimental-interpreter.h"
 #include "src/regexp/regexp-parser.h"
+#include "src/regexp/regexp-result-vector.h"
 #include "src/utils/ostreams.h"
 
-namespace v8 {
-namespace internal {
+namespace v8::internal {
 
-bool ExperimentalRegExp::CanBeHandled(RegExpTree* tree, Handle<String> pattern,
+bool ExperimentalRegExp::CanBeHandled(RegExpTree* tree,
+                                      DirectHandle<String> pattern,
                                       RegExpFlags flags, int capture_count) {
   DCHECK(v8_flags.enable_experimental_regexp_engine ||
          v8_flags.enable_experimental_regexp_engine_on_excessive_backtracks);
@@ -27,9 +30,9 @@ bool ExperimentalRegExp::CanBeHandled(RegExpTree* tree, Handle<String> pattern,
   return can_be_handled;
 }
 
-void ExperimentalRegExp::Initialize(Isolate* isolate, Handle<JSRegExp> re,
-                                    Handle<String> source, RegExpFlags flags,
-                                    int capture_count) {
+void ExperimentalRegExp::Initialize(Isolate* isolate, DirectHandle<JSRegExp> re,
+                                    DirectHandle<String> source,
+                                    RegExpFlags flags, int capture_count) {
   DCHECK(v8_flags.enable_experimental_regexp_engine);
   if (v8_flags.trace_experimental_regexp_engine) {
     StdoutStream{} << "Initializing experimental regexp " << *source
@@ -40,23 +43,26 @@ void ExperimentalRegExp::Initialize(Isolate* isolate, Handle<JSRegExp> re,
       re, source, JSRegExp::AsJSRegExpFlags(flags), capture_count);
 }
 
-bool ExperimentalRegExp::IsCompiled(Handle<JSRegExp> re, Isolate* isolate) {
+bool ExperimentalRegExp::IsCompiled(DirectHandle<IrRegExpData> re_data,
+                                    Isolate* isolate) {
   DCHECK(v8_flags.enable_experimental_regexp_engine);
-  DCHECK_EQ(re->type_tag(), JSRegExp::EXPERIMENTAL);
+  DCHECK_EQ(re_data->type_tag(), RegExpData::Type::EXPERIMENTAL);
 #ifdef VERIFY_HEAP
-  if (v8_flags.verify_heap) re->JSRegExpVerify(isolate);
+  if (v8_flags.verify_heap) re_data->IrRegExpDataVerify(isolate);
 #endif
 
   static constexpr bool kIsLatin1 = true;
-  return re->bytecode(kIsLatin1) != Smi::FromInt(JSRegExp::kUninitializedValue);
+  return re_data->has_bytecode(kIsLatin1);
 }
 
 template <class T>
-Handle<ByteArray> VectorToByteArray(Isolate* isolate, base::Vector<T> data) {
-  static_assert(std::is_trivial<T>::value);
+DirectHandle<TrustedByteArray> VectorToByteArray(Isolate* isolate,
+                                                 base::Vector<T> data) {
+  static_assert(std::is_trivial_v<T>);
 
   int byte_length = sizeof(T) * data.length();
-  Handle<ByteArray> byte_array = isolate->factory()->NewByteArray(byte_length);
+  DirectHandle<TrustedByteArray> byte_array =
+      isolate->factory()->NewTrustedByteArray(byte_length);
   DisallowGarbageCollection no_gc;
   MemCopy(byte_array->begin(), data.begin(), byte_length);
   return byte_array;
@@ -65,35 +71,35 @@ Handle<ByteArray> VectorToByteArray(Isolate* isolate, base::Vector<T> data) {
 namespace {
 
 struct CompilationResult {
-  Handle<ByteArray> bytecode;
-  Handle<FixedArray> capture_name_map;
+  DirectHandle<TrustedByteArray> bytecode;
+  DirectHandle<FixedArray> capture_name_map;
 };
 
 // Compiles source pattern, but doesn't change the regexp object.
-base::Optional<CompilationResult> CompileImpl(Isolate* isolate,
-                                              Handle<JSRegExp> regexp) {
+std::optional<CompilationResult> CompileImpl(
+    Isolate* isolate, DirectHandle<IrRegExpData> re_data) {
   Zone zone(isolate->allocator(), ZONE_NAME);
 
-  Handle<String> source(regexp->source(), isolate);
+  DirectHandle<String> source(re_data->source(), isolate);
 
   // Parse and compile the regexp source.
   RegExpCompileData parse_result;
   DCHECK(!isolate->has_exception());
 
-  RegExpFlags flags = JSRegExp::AsRegExpFlags(regexp->flags());
+  RegExpFlags flags = JSRegExp::AsRegExpFlags(re_data->flags());
   bool parse_success = RegExpParser::ParseRegExpFromHeapString(
       isolate, &zone, source, flags, &parse_result);
   if (!parse_success) {
     // The pattern was already parsed successfully during initialization, so
     // the only way parsing can fail now is because of stack overflow.
     DCHECK_EQ(parse_result.error, RegExpError::kStackOverflow);
-    USE(RegExp::ThrowRegExpException(isolate, regexp, flags, source,
+    USE(RegExp::ThrowRegExpException(isolate, flags, source,
                                      parse_result.error));
-    return base::nullopt;
+    return std::nullopt;
   }
 
   ZoneList<RegExpInstruction> bytecode = ExperimentalRegExpCompiler::Compile(
-      parse_result.tree, JSRegExp::AsRegExpFlags(regexp->flags()), &zone);
+      parse_result.tree, JSRegExp::AsRegExpFlags(re_data->flags()), &zone);
 
   CompilationResult result;
   result.bytecode = VectorToByteArray(isolate, bytecode.ToVector());
@@ -104,33 +110,34 @@ base::Optional<CompilationResult> CompileImpl(Isolate* isolate,
 
 }  // namespace
 
-bool ExperimentalRegExp::Compile(Isolate* isolate, Handle<JSRegExp> re) {
+bool ExperimentalRegExp::Compile(Isolate* isolate,
+                                 DirectHandle<IrRegExpData> re_data) {
   DCHECK(v8_flags.enable_experimental_regexp_engine);
-  DCHECK_EQ(re->type_tag(), JSRegExp::EXPERIMENTAL);
+  DCHECK_EQ(re_data->type_tag(), RegExpData::Type::EXPERIMENTAL);
 #ifdef VERIFY_HEAP
-  if (v8_flags.verify_heap) re->JSRegExpVerify(isolate);
+  if (v8_flags.verify_heap) re_data->IrRegExpDataVerify(isolate);
 #endif
 
-  Handle<String> source(re->source(), isolate);
+  DirectHandle<String> source(re_data->source(), isolate);
   if (v8_flags.trace_experimental_regexp_engine) {
     StdoutStream{} << "Compiling experimental regexp " << *source << std::endl;
   }
 
-  base::Optional<CompilationResult> compilation_result =
-      CompileImpl(isolate, re);
+  std::optional<CompilationResult> compilation_result =
+      CompileImpl(isolate, re_data);
   if (!compilation_result.has_value()) {
     DCHECK(isolate->has_exception());
     return false;
   }
 
-  re->set_bytecode_and_trampoline(isolate, compilation_result->bytecode);
-  re->set_capture_name_map(compilation_result->capture_name_map);
+  re_data->SetBytecodeForExperimental(isolate, *compilation_result->bytecode);
+  re_data->set_capture_name_map(compilation_result->capture_name_map);
 
   return true;
 }
 
 base::Vector<RegExpInstruction> AsInstructionSequence(
-    Tagged<ByteArray> raw_bytes) {
+    Tagged<TrustedByteArray> raw_bytes) {
   RegExpInstruction* inst_begin =
       reinterpret_cast<RegExpInstruction*>(raw_bytes->begin());
   int inst_num = raw_bytes->length() / sizeof(RegExpInstruction);
@@ -141,7 +148,7 @@ base::Vector<RegExpInstruction> AsInstructionSequence(
 namespace {
 
 int32_t ExecRawImpl(Isolate* isolate, RegExp::CallOrigin call_origin,
-                    Tagged<ByteArray> bytecode, Tagged<String> subject,
+                    Tagged<TrustedByteArray> bytecode, Tagged<String> subject,
                     int capture_count, int32_t* output_registers,
                     int32_t output_register_count, int32_t subject_index) {
   DisallowGarbageCollection no_gc;
@@ -163,30 +170,33 @@ int32_t ExecRawImpl(Isolate* isolate, RegExp::CallOrigin call_origin,
 }  // namespace
 
 // Returns the number of matches.
-int32_t ExperimentalRegExp::ExecRaw(
-    Isolate* isolate, RegExp::CallOrigin call_origin, Tagged<JSRegExp> regexp,
-    Tagged<String> subject, int32_t* output_registers,
-    int32_t output_register_count, int32_t subject_index) {
+int32_t ExperimentalRegExp::ExecRaw(Isolate* isolate,
+                                    RegExp::CallOrigin call_origin,
+                                    Tagged<IrRegExpData> regexp_data,
+                                    Tagged<String> subject,
+                                    int32_t* output_registers,
+                                    int32_t output_register_count,
+                                    int32_t subject_index) {
   CHECK(v8_flags.enable_experimental_regexp_engine);
   DisallowGarbageCollection no_gc;
 
   if (v8_flags.trace_experimental_regexp_engine) {
-    StdoutStream{} << "Executing experimental regexp " << regexp->source()
+    StdoutStream{} << "Executing experimental regexp " << regexp_data->source()
                    << std::endl;
   }
 
   static constexpr bool kIsLatin1 = true;
-  Tagged<ByteArray> bytecode = ByteArray::cast(regexp->bytecode(kIsLatin1));
+  Tagged<TrustedByteArray> bytecode = regexp_data->bytecode(kIsLatin1);
 
   return ExecRawImpl(isolate, call_origin, bytecode, subject,
-                     regexp->capture_count(), output_registers,
+                     regexp_data->capture_count(), output_registers,
                      output_register_count, subject_index);
 }
 
 int32_t ExperimentalRegExp::MatchForCallFromJs(
     Address subject, int32_t start_position, Address input_start,
     Address input_end, int* output_registers, int32_t output_register_count,
-    RegExp::CallOrigin call_origin, Isolate* isolate, Address regexp) {
+    RegExp::CallOrigin call_origin, Isolate* isolate, Address regexp_data) {
   DCHECK(v8_flags.enable_experimental_regexp_engine);
   DCHECK_NOT_NULL(isolate);
   DCHECK_NOT_NULL(output_registers);
@@ -197,61 +207,50 @@ int32_t ExperimentalRegExp::MatchForCallFromJs(
   DisallowHandleAllocation no_handles;
   DisallowHandleDereference no_deref;
 
-  Tagged<String> subject_string = String::cast(Tagged<Object>(subject));
+  Tagged<String> subject_string = Cast<String>(Tagged<Object>(subject));
 
-  Tagged<JSRegExp> regexp_obj = JSRegExp::cast(Tagged<Object>(regexp));
+  Tagged<IrRegExpData> regexp_data_obj =
+      Cast<IrRegExpData>(Tagged<Object>(regexp_data));
 
-  return ExecRaw(isolate, RegExp::kFromJs, regexp_obj, subject_string,
+  return ExecRaw(isolate, RegExp::kFromJs, regexp_data_obj, subject_string,
                  output_registers, output_register_count, start_position);
 }
 
-MaybeHandle<Object> ExperimentalRegExp::Exec(
-    Isolate* isolate, Handle<JSRegExp> regexp, Handle<String> subject,
-    int subject_index, Handle<RegExpMatchInfo> last_match_info,
-    RegExp::ExecQuirks exec_quirks) {
+// static
+std::optional<int> ExperimentalRegExp::Exec(
+    Isolate* isolate, DirectHandle<IrRegExpData> regexp_data,
+    DirectHandle<String> subject, int index, int32_t* result_offsets_vector,
+    uint32_t result_offsets_vector_length) {
   DCHECK(v8_flags.enable_experimental_regexp_engine);
-  DCHECK_EQ(regexp->type_tag(), JSRegExp::EXPERIMENTAL);
+  DCHECK_EQ(regexp_data->type_tag(), RegExpData::Type::EXPERIMENTAL);
 #ifdef VERIFY_HEAP
-  if (v8_flags.verify_heap) regexp->JSRegExpVerify(isolate);
+  if (v8_flags.verify_heap) regexp_data->IrRegExpDataVerify(isolate);
 #endif
 
-  if (!IsCompiled(regexp, isolate) && !Compile(isolate, regexp)) {
+  if (!IsCompiled(regexp_data, isolate) && !Compile(isolate, regexp_data)) {
     DCHECK(isolate->has_exception());
-    return MaybeHandle<Object>();
+    return {};
   }
 
-  DCHECK(IsCompiled(regexp, isolate));
+  DCHECK(IsCompiled(regexp_data, isolate));
 
   subject = String::Flatten(isolate, subject);
 
-  int capture_count = regexp->capture_count();
-  int output_register_count = JSRegExp::RegistersForCaptureCount(capture_count);
-
-  int32_t* output_registers;
-  std::unique_ptr<int32_t[]> output_registers_release;
-  if (output_register_count <= Isolate::kJSRegexpStaticOffsetsVectorSize) {
-    output_registers = isolate->jsregexp_static_offsets_vector();
-  } else {
-    output_registers = NewArray<int32_t>(output_register_count);
-    output_registers_release.reset(output_registers);
-  }
+  DCHECK_GE(result_offsets_vector_length,
+            JSRegExp::RegistersForCaptureCount(regexp_data->capture_count()));
 
   do {
     int num_matches =
-        ExecRaw(isolate, RegExp::kFromRuntime, *regexp, *subject,
-                output_registers, output_register_count, subject_index);
+        ExecRaw(isolate, RegExp::kFromRuntime, *regexp_data, *subject,
+                result_offsets_vector, result_offsets_vector_length, index);
 
     if (num_matches > 0) {
-      DCHECK_EQ(num_matches, 1);
-      if (exec_quirks == RegExp::ExecQuirks::kTreatMatchAtEndAsFailure) {
-        if (output_registers[0] >= subject->length()) {
-          return isolate->factory()->null_value();
-        }
-      }
-      return RegExp::SetLastMatchInfo(isolate, last_match_info, subject,
-                                      capture_count, output_registers);
+      DCHECK_LE(num_matches * JSRegExp::RegistersForCaptureCount(
+                                  regexp_data->capture_count()),
+                result_offsets_vector_length);
+      return num_matches;
     } else if (num_matches == 0) {
-      return isolate->factory()->null_value();
+      return num_matches;
     } else {
       DCHECK_LT(num_matches, 0);
       if (num_matches == RegExp::kInternalRegExpRetry) {
@@ -259,70 +258,52 @@ MaybeHandle<Object> ExperimentalRegExp::Exec(
         continue;
       }
       DCHECK(isolate->has_exception());
-      return MaybeHandle<Object>();
+      return {};
     }
   } while (true);
   UNREACHABLE();
 }
 
-int32_t ExperimentalRegExp::OneshotExecRaw(Isolate* isolate,
-                                           Handle<JSRegExp> regexp,
-                                           Handle<String> subject,
-                                           int32_t* output_registers,
-                                           int32_t output_register_count,
-                                           int32_t subject_index) {
+int32_t ExperimentalRegExp::OneshotExecRaw(
+    Isolate* isolate, DirectHandle<IrRegExpData> regexp_data,
+    DirectHandle<String> subject, int32_t* output_registers,
+    int32_t output_register_count, int32_t subject_index) {
   CHECK(v8_flags.enable_experimental_regexp_engine_on_excessive_backtracks);
 
   if (v8_flags.trace_experimental_regexp_engine) {
     StdoutStream{} << "Experimental execution (oneshot) of regexp "
-                   << regexp->source() << std::endl;
+                   << regexp_data->source() << std::endl;
   }
 
-  base::Optional<CompilationResult> compilation_result =
-      CompileImpl(isolate, regexp);
+  std::optional<CompilationResult> compilation_result =
+      CompileImpl(isolate, regexp_data);
   if (!compilation_result.has_value()) return RegExp::kInternalRegExpException;
 
   DisallowGarbageCollection no_gc;
   return ExecRawImpl(isolate, RegExp::kFromRuntime,
                      *compilation_result->bytecode, *subject,
-                     regexp->capture_count(), output_registers,
+                     regexp_data->capture_count(), output_registers,
                      output_register_count, subject_index);
 }
 
-MaybeHandle<Object> ExperimentalRegExp::OneshotExec(
-    Isolate* isolate, Handle<JSRegExp> regexp, Handle<String> subject,
-    int subject_index, Handle<RegExpMatchInfo> last_match_info,
-    RegExp::ExecQuirks exec_quirks) {
+std::optional<int> ExperimentalRegExp::OneshotExec(
+    Isolate* isolate, DirectHandle<IrRegExpData> regexp_data,
+    DirectHandle<String> subject, int subject_index,
+    int32_t* result_offsets_vector, uint32_t result_offsets_vector_length) {
   DCHECK(v8_flags.enable_experimental_regexp_engine_on_excessive_backtracks);
-  DCHECK_NE(regexp->type_tag(), JSRegExp::NOT_COMPILED);
-
-  int capture_count = regexp->capture_count();
-  int output_register_count = JSRegExp::RegistersForCaptureCount(capture_count);
-
-  int32_t* output_registers;
-  std::unique_ptr<int32_t[]> output_registers_release;
-  if (output_register_count <= Isolate::kJSRegexpStaticOffsetsVectorSize) {
-    output_registers = isolate->jsregexp_static_offsets_vector();
-  } else {
-    output_registers = NewArray<int32_t>(output_register_count);
-    output_registers_release.reset(output_registers);
-  }
 
   do {
-    int num_matches = OneshotExecRaw(isolate, regexp, subject, output_registers,
-                                     output_register_count, subject_index);
+    int num_matches =
+        OneshotExecRaw(isolate, regexp_data, subject, result_offsets_vector,
+                       result_offsets_vector_length, subject_index);
 
     if (num_matches > 0) {
-      DCHECK_EQ(num_matches, 1);
-      if (exec_quirks == RegExp::ExecQuirks::kTreatMatchAtEndAsFailure) {
-        if (output_registers[0] >= subject->length()) {
-          return isolate->factory()->null_value();
-        }
-      }
-      return RegExp::SetLastMatchInfo(isolate, last_match_info, subject,
-                                      capture_count, output_registers);
+      DCHECK_LE(num_matches * JSRegExp::RegistersForCaptureCount(
+                                  regexp_data->capture_count()),
+                result_offsets_vector_length);
+      return num_matches;
     } else if (num_matches == 0) {
-      return isolate->factory()->null_value();
+      return num_matches;
     } else {
       DCHECK_LT(num_matches, 0);
       if (num_matches == RegExp::kInternalRegExpRetry) {
@@ -330,11 +311,10 @@ MaybeHandle<Object> ExperimentalRegExp::OneshotExec(
         continue;
       }
       DCHECK(isolate->has_exception());
-      return MaybeHandle<Object>();
+      return {};
     }
   } while (true);
   UNREACHABLE();
 }
 
-}  // namespace internal
-}  // namespace v8
+}  // namespace v8::internal
