@@ -58,7 +58,7 @@ class RootVisitor;
   V(Map, free_space_map, FreeSpaceMap)                                         \
   V(Map, one_pointer_filler_map, OnePointerFillerMap)                          \
   V(Map, two_pointer_filler_map, TwoPointerFillerMap)                          \
-  V(Oddball, uninitialized_value, UninitializedValue)                          \
+  V(Hole, uninitialized_value, UninitializedValue)                             \
   V(Undefined, undefined_value, UndefinedValue)                                \
   V(Hole, the_hole_value, TheHoleValue)                                        \
   V(Null, null_value, NullValue)                                               \
@@ -89,14 +89,15 @@ class RootVisitor;
   V(FixedArray, empty_fixed_array, EmptyFixedArray)                            \
   V(DescriptorArray, empty_descriptor_array, EmptyDescriptorArray)             \
   /* Entries beyond the first 32                                            */ \
-  /* Oddballs */                                                               \
-  V(Oddball, arguments_marker, ArgumentsMarker)                                \
-  V(Oddball, exception, Exception)                                             \
-  V(Oddball, termination_exception, TerminationException)                      \
-  V(Oddball, optimized_out, OptimizedOut)                                      \
-  V(Oddball, stale_register, StaleRegister)                                    \
   /* Holes */                                                                  \
+  V(Hole, arguments_marker, ArgumentsMarker)                                   \
+  V(Hole, exception, Exception)                                                \
+  V(Hole, termination_exception, TerminationException)                         \
+  V(Hole, optimized_out, OptimizedOut)                                         \
+  V(Hole, stale_register, StaleRegister)                                       \
   V(Hole, property_cell_hole_value, PropertyCellHoleValue)                     \
+  V(Hole, hash_table_hole_value, HashTableHoleValue)                           \
+  V(Hole, promise_hole_value, PromiseHoleValue)                                \
   /* Maps */                                                                   \
   V(Map, script_context_table_map, ScriptContextTableMap)                      \
   V(Map, closure_feedback_cell_array_map, ClosureFeedbackCellArrayMap)         \
@@ -107,7 +108,10 @@ class RootVisitor;
   V(Map, bytecode_array_map, BytecodeArrayMap)                                 \
   V(Map, code_map, CodeMap)                                                    \
   V(Map, coverage_info_map, CoverageInfoMap)                                   \
+  V(Map, dictionary_template_info_map, DictionaryTemplateInfoMap)              \
   V(Map, global_dictionary_map, GlobalDictionaryMap)                           \
+  V(Map, global_context_side_property_cell_map,                                \
+    GlobalContextSidePropertyCellMap)                                          \
   V(Map, many_closures_cell_map, ManyClosuresCellMap)                          \
   V(Map, mega_dom_handler_map, MegaDomHandlerMap)                              \
   V(Map, module_info_map, ModuleInfoMap)                                       \
@@ -123,9 +127,10 @@ class RootVisitor;
   V(Map, preparse_data_map, PreparseDataMap)                                   \
   V(Map, property_array_map, PropertyArrayMap)                                 \
   V(Map, accessor_info_map, AccessorInfoMap)                                   \
-  V(Map, side_effect_call_handler_info_map, SideEffectCallHandlerInfoMap)      \
-  V(Map, side_effect_free_call_handler_info_map,                               \
-    SideEffectFreeCallHandlerInfoMap)                                          \
+  V(Map, regexp_match_info_map, RegExpMatchInfoMap)                            \
+  V(Map, regexp_data_map, RegExpDataMap)                                       \
+  V(Map, atom_regexp_data_map, AtomRegExpDataMap)                              \
+  V(Map, ir_regexp_data_map, IrRegExpDataMap)                                  \
   V(Map, simple_number_dictionary_map, SimpleNumberDictionaryMap)              \
   V(Map, small_ordered_hash_map_map, SmallOrderedHashMapMap)                   \
   V(Map, small_ordered_hash_set_map, SmallOrderedHashSetMap)                   \
@@ -133,22 +138,33 @@ class RootVisitor;
   V(Map, source_text_module_map, SourceTextModuleMap)                          \
   V(Map, swiss_name_dictionary_map, SwissNameDictionaryMap)                    \
   V(Map, synthetic_module_map, SyntheticModuleMap)                             \
-  IF_WASM(V, Map, wasm_api_function_ref_map, WasmApiFunctionRefMap)            \
+  IF_WASM(V, Map, wasm_import_data_map, WasmImportDataMap)                     \
   IF_WASM(V, Map, wasm_capi_function_data_map, WasmCapiFunctionDataMap)        \
+  IF_WASM(V, Map, wasm_continuation_object_map, WasmContinuationObjectMap)     \
+  IF_WASM(V, Map, wasm_dispatch_table_map, WasmDispatchTableMap)               \
   IF_WASM(V, Map, wasm_exported_function_data_map,                             \
           WasmExportedFunctionDataMap)                                         \
   IF_WASM(V, Map, wasm_internal_function_map, WasmInternalFunctionMap)         \
+  IF_WASM(V, Map, wasm_func_ref_map, WasmFuncRefMap)                           \
   IF_WASM(V, Map, wasm_js_function_data_map, WasmJSFunctionDataMap)            \
-  IF_WASM(V, Map, wasm_resume_data_map, WasmResumeDataMap)                     \
-  IF_WASM(V, Map, wasm_type_info_map, WasmTypeInfoMap)                         \
-  IF_WASM(V, Map, wasm_continuation_object_map, WasmContinuationObjectMap)     \
   IF_WASM(V, Map, wasm_null_map, WasmNullMap)                                  \
+  IF_WASM(V, Map, wasm_resume_data_map, WasmResumeDataMap)                     \
+  IF_WASM(V, Map, wasm_suspender_object_map, WasmSuspenderObjectMap)           \
+  IF_WASM(V, Map, wasm_trusted_instance_data_map, WasmTrustedInstanceDataMap)  \
+  IF_WASM(V, Map, wasm_type_info_map, WasmTypeInfoMap)                         \
   V(Map, weak_fixed_array_map, WeakFixedArrayMap)                              \
   V(Map, weak_array_list_map, WeakArrayListMap)                                \
   V(Map, ephemeron_hash_table_map, EphemeronHashTableMap)                      \
   V(Map, embedder_data_array_map, EmbedderDataArrayMap)                        \
   V(Map, weak_cell_map, WeakCellMap)                                           \
-  V(Map, external_pointer_array_map, ExternalPointerArrayMap)                  \
+  V(Map, trusted_fixed_array_map, TrustedFixedArrayMap)                        \
+  V(Map, trusted_weak_fixed_array_map, TrustedWeakFixedArrayMap)               \
+  V(Map, trusted_byte_array_map, TrustedByteArrayMap)                          \
+  V(Map, protected_fixed_array_map, ProtectedFixedArrayMap)                    \
+  V(Map, protected_weak_fixed_array_map, ProtectedWeakFixedArrayMap)           \
+  V(Map, interpreter_data_map, InterpreterDataMap)                             \
+  V(Map, shared_function_info_wrapper_map, SharedFunctionInfoWrapperMap)       \
+  V(Map, trusted_foreign_map, TrustedForeignMap)                               \
   /* String maps */                                                            \
   V(Map, seq_two_byte_string_map, SeqTwoByteStringMap)                         \
   V(Map, cons_two_byte_string_map, ConsTwoByteStringMap)                       \
@@ -182,17 +198,9 @@ class RootVisitor;
     SharedUncachedExternalTwoByteStringMap)                                    \
   /* Oddball maps */                                                           \
   V(Map, undefined_map, UndefinedMap)                                          \
-  V(Map, hole_map, HoleMap)                                                    \
   V(Map, null_map, NullMap)                                                    \
   V(Map, boolean_map, BooleanMap)                                              \
-  V(Map, uninitialized_map, UninitializedMap)                                  \
-  V(Map, arguments_marker_map, ArgumentsMarkerMap)                             \
-  V(Map, exception_map, ExceptionMap)                                          \
-  V(Map, termination_exception_map, TerminationExceptionMap)                   \
-  V(Map, optimized_out_map, OptimizedOutMap)                                   \
-  V(Map, stale_register_map, StaleRegisterMap)                                 \
-  V(Map, self_reference_marker_map, SelfReferenceMarkerMap)                    \
-  V(Map, basic_block_counters_marker_map, BasicBlockCountersMarkerMap)         \
+  V(Map, hole_map, HoleMap)                                                    \
   /* Shared space object maps */                                               \
   V(Map, js_shared_array_map, JSSharedArrayMap)                                \
   V(Map, js_atomics_mutex_map, JSAtomicsMutexMap)                              \
@@ -201,8 +209,6 @@ class RootVisitor;
   V(EnumCache, empty_enum_cache, EmptyEnumCache)                               \
   V(PropertyArray, empty_property_array, EmptyPropertyArray)                   \
   V(ByteArray, empty_byte_array, EmptyByteArray)                               \
-  V(ExternalPointerArray, empty_external_pointer_array,                        \
-    EmptyExternalPointerArray)                                                 \
   V(ObjectBoilerplateDescription, empty_object_boilerplate_description,        \
     EmptyObjectBoilerplateDescription)                                         \
   V(ArrayBoilerplateDescription, empty_array_boilerplate_description,          \
@@ -224,13 +230,14 @@ class RootVisitor;
   V(WeakFixedArray, empty_weak_fixed_array, EmptyWeakFixedArray)               \
   V(WeakArrayList, empty_weak_array_list, EmptyWeakArrayList)                  \
   V(Cell, invalid_prototype_validity_cell, InvalidPrototypeValidityCell)       \
+  V(FeedbackCell, many_closures_cell, ManyClosuresCell)                        \
   STRONG_READ_ONLY_HEAP_NUMBER_ROOT_LIST(V)                                    \
   /* Table of strings of one-byte single characters */                         \
   V(FixedArray, single_character_string_table, SingleCharacterStringTable)     \
   /* Marker for self-references during code-generation */                      \
-  V(HeapObject, self_reference_marker, SelfReferenceMarker)                    \
+  V(Hole, self_reference_marker, SelfReferenceMarker)                          \
   /* Marker for basic-block usage counters array during code-generation */     \
-  V(Oddball, basic_block_counters_marker, BasicBlockCountersMarker)            \
+  V(Hole, basic_block_counters_marker, BasicBlockCountersMarker)               \
   /* Canonical scope infos */                                                  \
   V(ScopeInfo, global_this_binding_scope_info, GlobalThisBindingScopeInfo)     \
   V(ScopeInfo, empty_function_scope_info, EmptyFunctionScopeInfo)              \
@@ -242,23 +249,92 @@ class RootVisitor;
   IF_WASM(V, HeapObject, wasm_null_padding, WasmNullPadding)                   \
   IF_WASM(V, WasmNull, wasm_null, WasmNull)
 
+// TODO(saelo): ideally, these would be read-only roots (and then become part
+// of the READ_ONLY_ROOT_LIST instead of the
+// STRONG_MUTABLE_IMMOVABLE_ROOT_LIST). However, currently we do not have a
+// trusted RO space.
+#define TRUSTED_ROOT_LIST(V)                                              \
+  V(TrustedByteArray, empty_trusted_byte_array, EmptyTrustedByteArray)    \
+  V(TrustedFixedArray, empty_trusted_fixed_array, EmptyTrustedFixedArray) \
+  V(TrustedWeakFixedArray, empty_trusted_weak_fixed_array,                \
+    EmptyTrustedWeakFixedArray)                                           \
+  V(ProtectedFixedArray, empty_protected_fixed_array,                     \
+    EmptyProtectedFixedArray)                                             \
+  V(ProtectedWeakFixedArray, empty_protected_weak_fixed_array,            \
+    EmptyProtectedWeakFixedArray)
+
+#define BUILTINS_WITH_SFI_LIST_GENERATOR(APPLY, V)                             \
+  APPLY(V, ProxyRevoke, proxy_revoke)                                          \
+  APPLY(V, AsyncFromSyncIteratorCloseSyncAndRethrow,                           \
+        async_from_sync_iterator_close_sync_and_rethrow)                       \
+  APPLY(V, AsyncFunctionAwaitRejectClosure,                                    \
+        async_function_await_reject_closure)                                   \
+  APPLY(V, AsyncFunctionAwaitResolveClosure,                                   \
+        async_function_await_resolve_closure)                                  \
+  APPLY(V, AsyncGeneratorAwaitRejectClosure,                                   \
+        async_generator_await_reject_closure)                                  \
+  APPLY(V, AsyncGeneratorAwaitResolveClosure,                                  \
+        async_generator_await_resolve_closure)                                 \
+  APPLY(V, AsyncGeneratorYieldWithAwaitResolveClosure,                         \
+        async_generator_yield_with_await_resolve_closure)                      \
+  APPLY(V, AsyncGeneratorReturnClosedResolveClosure,                           \
+        async_generator_return_closed_resolve_closure)                         \
+  APPLY(V, AsyncGeneratorReturnClosedRejectClosure,                            \
+        async_generator_return_closed_reject_closure)                          \
+  APPLY(V, AsyncGeneratorReturnResolveClosure,                                 \
+        async_generator_return_resolve_closure)                                \
+  APPLY(V, AsyncIteratorValueUnwrap, async_iterator_value_unwrap)              \
+  APPLY(V, ArrayFromAsyncArrayLikeOnFulfilled,                                 \
+        array_from_async_array_like_on_fulfilled)                              \
+  APPLY(V, ArrayFromAsyncArrayLikeOnRejected,                                  \
+        array_from_async_array_like_on_rejected)                               \
+  APPLY(V, ArrayFromAsyncIterableOnFulfilled,                                  \
+        array_from_async_iterable_on_fulfilled)                                \
+  APPLY(V, ArrayFromAsyncIterableOnRejected,                                   \
+        array_from_async_iterable_on_rejected)                                 \
+  APPLY(V, PromiseCapabilityDefaultResolve,                                    \
+        promise_capability_default_resolve)                                    \
+  APPLY(V, PromiseCapabilityDefaultReject, promise_capability_default_reject)  \
+  APPLY(V, PromiseGetCapabilitiesExecutor, promise_get_capabilities_executor)  \
+  APPLY(V, PromiseAllSettledResolveElementClosure,                             \
+        promise_all_settled_resolve_element_closure)                           \
+  APPLY(V, PromiseAllSettledRejectElementClosure,                              \
+        promise_all_settled_reject_element_closure)                            \
+  APPLY(V, PromiseAllResolveElementClosure,                                    \
+        promise_all_resolve_element_closure)                                   \
+  APPLY(V, PromiseAnyRejectElementClosure, promise_any_reject_element_closure) \
+  APPLY(V, PromiseThrowerFinally, promise_thrower_finally)                     \
+  APPLY(V, PromiseValueThunkFinally, promise_value_thunk_finally)              \
+  APPLY(V, PromiseThenFinally, promise_then_finally)                           \
+  APPLY(V, PromiseCatchFinally, promise_catch_finally)                         \
+  APPLY(V, ShadowRealmImportValueFulfilled,                                    \
+        shadow_realm_import_value_fulfilled)                                   \
+  APPLY(V, AsyncIteratorPrototypeAsyncDisposeResolveClosure,                   \
+        async_iterator_prototype_async_dispose_resolve_closure)
+
+#define BUILTINS_WITH_SFI_ROOTS_LIST_ADAPTER(V, CamelName, underscore_name, \
+                                             ...)                           \
+  V(SharedFunctionInfo, underscore_name##_shared_fun, CamelName##SharedFun)
+
+#define BUILTINS_WITH_SFI_ROOTS_LIST(V) \
+  BUILTINS_WITH_SFI_LIST_GENERATOR(BUILTINS_WITH_SFI_ROOTS_LIST_ADAPTER, V)
+
 // Mutable roots that are known to be immortal immovable, for which we can
 // safely skip write barriers.
 #define STRONG_MUTABLE_IMMOVABLE_ROOT_LIST(V)                                  \
   ACCESSOR_INFO_ROOT_LIST(V)                                                   \
-  /* Maps */                                                                   \
-  V(Map, external_map, ExternalMap)                                            \
-  V(Map, message_object_map, JSMessageObjectMap)                               \
   /* Canonical empty values */                                                 \
   V(Script, empty_script, EmptyScript)                                         \
-  V(FeedbackCell, many_closures_cell, ManyClosuresCell)                        \
   /* Protectors */                                                             \
   V(PropertyCell, array_constructor_protector, ArrayConstructorProtector)      \
   V(PropertyCell, no_elements_protector, NoElementsProtector)                  \
   V(PropertyCell, mega_dom_protector, MegaDOMProtector)                        \
   V(PropertyCell, no_profiling_protector, NoProfilingProtector)                \
+  V(PropertyCell, no_undetectable_objects_protector,                           \
+    NoUndetectableObjectsProtector)                                            \
   V(PropertyCell, is_concat_spreadable_protector, IsConcatSpreadableProtector) \
   V(PropertyCell, array_species_protector, ArraySpeciesProtector)              \
+  V(PropertyCell, typed_array_length_protector, TypedArrayLengthProtector)     \
   V(PropertyCell, typed_array_species_protector, TypedArraySpeciesProtector)   \
   V(PropertyCell, promise_species_protector, PromiseSpeciesProtector)          \
   V(PropertyCell, regexp_species_protector, RegExpSpeciesProtector)            \
@@ -272,61 +348,35 @@ class RootVisitor;
   V(PropertyCell, promise_then_protector, PromiseThenProtector)                \
   V(PropertyCell, set_iterator_protector, SetIteratorProtector)                \
   V(PropertyCell, string_iterator_protector, StringIteratorProtector)          \
+  V(PropertyCell, string_wrapper_to_primitive_protector,                       \
+    StringWrapperToPrimitiveProtector)                                         \
   V(PropertyCell, number_string_not_regexp_like_protector,                     \
     NumberStringNotRegexpLikeProtector)                                        \
   /* Caches */                                                                 \
   V(FixedArray, string_split_cache, StringSplitCache)                          \
   V(FixedArray, regexp_multiple_cache, RegExpMultipleCache)                    \
+  V(FixedArray, regexp_match_global_atom_cache, RegExpMatchGlobalAtomCache)    \
   /* Indirection lists for isolate-independent builtins */                     \
   V(FixedArray, builtins_constants_table, BuiltinsConstantsTable)              \
   /* Internal SharedFunctionInfos */                                           \
-  V(SharedFunctionInfo, async_function_await_reject_shared_fun,                \
-    AsyncFunctionAwaitRejectSharedFun)                                         \
-  V(SharedFunctionInfo, async_function_await_resolve_shared_fun,               \
-    AsyncFunctionAwaitResolveSharedFun)                                        \
-  V(SharedFunctionInfo, async_generator_await_reject_shared_fun,               \
-    AsyncGeneratorAwaitRejectSharedFun)                                        \
-  V(SharedFunctionInfo, async_generator_await_resolve_shared_fun,              \
-    AsyncGeneratorAwaitResolveSharedFun)                                       \
-  V(SharedFunctionInfo, async_generator_yield_with_await_resolve_shared_fun,   \
-    AsyncGeneratorYieldWithAwaitResolveSharedFun)                              \
-  V(SharedFunctionInfo, async_generator_return_resolve_shared_fun,             \
-    AsyncGeneratorReturnResolveSharedFun)                                      \
-  V(SharedFunctionInfo, async_generator_return_closed_reject_shared_fun,       \
-    AsyncGeneratorReturnClosedRejectSharedFun)                                 \
-  V(SharedFunctionInfo, async_generator_return_closed_resolve_shared_fun,      \
-    AsyncGeneratorReturnClosedResolveSharedFun)                                \
-  V(SharedFunctionInfo, async_iterator_value_unwrap_shared_fun,                \
-    AsyncIteratorValueUnwrapSharedFun)                                         \
-  V(SharedFunctionInfo, promise_all_resolve_element_shared_fun,                \
-    PromiseAllResolveElementSharedFun)                                         \
-  V(SharedFunctionInfo, promise_all_settled_resolve_element_shared_fun,        \
-    PromiseAllSettledResolveElementSharedFun)                                  \
-  V(SharedFunctionInfo, promise_all_settled_reject_element_shared_fun,         \
-    PromiseAllSettledRejectElementSharedFun)                                   \
-  V(SharedFunctionInfo, promise_any_reject_element_shared_fun,                 \
-    PromiseAnyRejectElementSharedFun)                                          \
-  V(SharedFunctionInfo, promise_capability_default_reject_shared_fun,          \
-    PromiseCapabilityDefaultRejectSharedFun)                                   \
-  V(SharedFunctionInfo, promise_capability_default_resolve_shared_fun,         \
-    PromiseCapabilityDefaultResolveSharedFun)                                  \
-  V(SharedFunctionInfo, promise_catch_finally_shared_fun,                      \
-    PromiseCatchFinallySharedFun)                                              \
-  V(SharedFunctionInfo, promise_get_capabilities_executor_shared_fun,          \
-    PromiseGetCapabilitiesExecutorSharedFun)                                   \
-  V(SharedFunctionInfo, promise_then_finally_shared_fun,                       \
-    PromiseThenFinallySharedFun)                                               \
-  V(SharedFunctionInfo, promise_thrower_finally_shared_fun,                    \
-    PromiseThrowerFinallySharedFun)                                            \
-  V(SharedFunctionInfo, promise_value_thunk_finally_shared_fun,                \
-    PromiseValueThunkFinallySharedFun)                                         \
-  V(SharedFunctionInfo, proxy_revoke_shared_fun, ProxyRevokeSharedFun)         \
-  V(SharedFunctionInfo, shadow_realm_import_value_fulfilled_sfi,               \
-    ShadowRealmImportValueFulfilledSFI)                                        \
   V(SharedFunctionInfo, source_text_module_execute_async_module_fulfilled_sfi, \
     SourceTextModuleExecuteAsyncModuleFulfilledSFI)                            \
   V(SharedFunctionInfo, source_text_module_execute_async_module_rejected_sfi,  \
-    SourceTextModuleExecuteAsyncModuleRejectedSFI)
+    SourceTextModuleExecuteAsyncModuleRejectedSFI)                             \
+  V(SharedFunctionInfo, atomics_mutex_async_unlock_resolve_handler_sfi,        \
+    AtomicsMutexAsyncUnlockResolveHandlerSFI)                                  \
+  V(SharedFunctionInfo, atomics_mutex_async_unlock_reject_handler_sfi,         \
+    AtomicsMutexAsyncUnlockRejectHandlerSFI)                                   \
+  V(SharedFunctionInfo, atomics_condition_acquire_lock_sfi,                    \
+    AtomicsConditionAcquireLockSFI)                                            \
+  V(SharedFunctionInfo, async_disposable_stack_on_fulfilled_shared_fun,        \
+    AsyncDisposableStackOnFulfilledSharedFun)                                  \
+  V(SharedFunctionInfo, async_disposable_stack_on_rejected_shared_fun,         \
+    AsyncDisposableStackOnRejectedSharedFun)                                   \
+  V(SharedFunctionInfo, async_dispose_from_sync_dispose_shared_fun,            \
+    AsyncDisposeFromSyncDisposeSharedFun)                                      \
+  BUILTINS_WITH_SFI_ROOTS_LIST(V)                                              \
+  TRUSTED_ROOT_LIST(V)
 
 // These root references can be updated by the mutator.
 #define STRONG_MUTABLE_MOVABLE_ROOT_LIST(V)                                 \
@@ -339,11 +389,10 @@ class RootVisitor;
   V(WeakArrayList, script_list, ScriptList)                                 \
   V(FixedArray, materialized_objects, MaterializedObjects)                  \
   V(WeakArrayList, detached_contexts, DetachedContexts)                     \
-  V(WeakArrayList, retaining_path_targets, RetainingPathTargets)            \
   /* Feedback vectors that we need for code coverage or type profile */     \
   V(Object, feedback_vectors_for_profiling_tools,                           \
     FeedbackVectorsForProfilingTools)                                       \
-  V(FixedArray, serialized_objects, SerializedObjects)                      \
+  V(HeapObject, serialized_objects, SerializedObjects)                      \
   V(FixedArray, serialized_global_proxy_sizes, SerializedGlobalProxySizes)  \
   V(ArrayList, message_listeners, MessageListeners)                         \
   /* Support for async stack traces */                                      \
@@ -358,8 +407,8 @@ class RootVisitor;
   V(HeapObject, locals_block_list_cache, DebugLocalsBlockListCache)         \
   IF_WASM(V, HeapObject, active_continuation, ActiveContinuation)           \
   IF_WASM(V, HeapObject, active_suspender, ActiveSuspender)                 \
-  IF_WASM(V, WeakArrayList, js_to_wasm_wrappers, JSToWasmWrappers)          \
-  IF_WASM(V, WeakArrayList, wasm_canonical_rtts, WasmCanonicalRtts)         \
+  IF_WASM(V, WeakFixedArray, js_to_wasm_wrappers, JSToWasmWrappers)         \
+  IF_WASM(V, WeakFixedArray, wasm_canonical_rtts, WasmCanonicalRtts)        \
   /* Internal SharedFunctionInfos */                                        \
   V(FunctionTemplateInfo, error_stack_getter_fun_template,                  \
     ErrorStackGetterSharedFun)                                              \
@@ -370,6 +419,7 @@ class RootVisitor;
 #define SMI_ROOT_LIST(V)                                                       \
   V(Smi, last_script_id, LastScriptId)                                         \
   V(Smi, last_debugging_id, LastDebuggingId)                                   \
+  V(Smi, last_stack_trace_id, LastStackTraceId)                                \
   /* To distinguish the function templates, so that we can find them in the */ \
   /* function cache of the native context. */                                  \
   V(Smi, next_template_serial_number, NextTemplateSerialNumber)                \
@@ -377,6 +427,8 @@ class RootVisitor;
     ConstructStubCreateDeoptPCOffset)                                          \
   V(Smi, construct_stub_invoke_deopt_pc_offset,                                \
     ConstructStubInvokeDeoptPCOffset)                                          \
+  V(Smi, deopt_pc_offset_after_adapt_shadow_stack,                             \
+    DeoptPCOffsetAfterAdaptShadowStack)                                        \
   V(Smi, interpreter_entry_return_pc_offset, InterpreterEntryReturnPCOffset)
 
 // Produces (String, name, CamelCase) entries
@@ -398,10 +450,11 @@ class RootVisitor;
   WELL_KNOWN_SYMBOL_LIST_GENERATOR(SYMBOL_ROOT_LIST_ADAPTER, V)
 
 // Produces (Na,e, name, CamelCase) entries
-#define NAME_FOR_PROTECTOR_ROOT_LIST(V)                            \
-  INTERNALIZED_STRING_FOR_PROTECTOR_LIST_GENERATOR(                \
-      INTERNALIZED_STRING_LIST_ADAPTER, V)                         \
-  SYMBOL_FOR_PROTECTOR_LIST_GENERATOR(SYMBOL_ROOT_LIST_ADAPTER, V) \
+#define NAME_FOR_PROTECTOR_ROOT_LIST(V)                                   \
+  INTERNALIZED_STRING_FOR_PROTECTOR_LIST_GENERATOR(                       \
+      INTERNALIZED_STRING_LIST_ADAPTER, V)                                \
+  SYMBOL_FOR_PROTECTOR_LIST_GENERATOR(SYMBOL_ROOT_LIST_ADAPTER, V)        \
+  PUBLIC_SYMBOL_FOR_PROTECTOR_LIST_GENERATOR(SYMBOL_ROOT_LIST_ADAPTER, V) \
   WELL_KNOWN_SYMBOL_FOR_PROTECTOR_LIST_GENERATOR(SYMBOL_ROOT_LIST_ADAPTER, V)
 
 // Adapts one ACCESSOR_INFO_LIST_GENERATOR entry to the ROOT_LIST-compatible
@@ -413,17 +466,20 @@ class RootVisitor;
 #define ACCESSOR_INFO_ROOT_LIST(V) \
   ACCESSOR_INFO_LIST_GENERATOR(ACCESSOR_INFO_ROOT_LIST_ADAPTER, V)
 
-#define READ_ONLY_ROOT_LIST(V)     \
-  STRONG_READ_ONLY_ROOT_LIST(V)    \
-  INTERNALIZED_STRING_ROOT_LIST(V) \
-  PRIVATE_SYMBOL_ROOT_LIST(V)      \
-  PUBLIC_SYMBOL_ROOT_LIST(V)       \
-  WELL_KNOWN_SYMBOL_ROOT_LIST(V)   \
-  STRUCT_MAPS_LIST(V)              \
-  TORQUE_DEFINED_MAP_ROOT_LIST(V)  \
-  ALLOCATION_SITE_MAPS_LIST(V)     \
-  NAME_FOR_PROTECTOR_ROOT_LIST(V)  \
-  DATA_HANDLER_MAPS_LIST(V)
+#define READ_ONLY_ROOT_LIST(V)      \
+  STRONG_READ_ONLY_ROOT_LIST(V)     \
+  INTERNALIZED_STRING_ROOT_LIST(V)  \
+  PRIVATE_SYMBOL_ROOT_LIST(V)       \
+  PUBLIC_SYMBOL_ROOT_LIST(V)        \
+  WELL_KNOWN_SYMBOL_ROOT_LIST(V)    \
+  STRUCT_MAPS_LIST(V)               \
+  TORQUE_DEFINED_MAP_ROOT_LIST(V)   \
+  ALLOCATION_SITE_MAPS_LIST(V)      \
+  NAME_FOR_PROTECTOR_ROOT_LIST(V)   \
+  DATA_HANDLER_MAPS_LIST(V)         \
+  /* Maps */                        \
+  V(Map, external_map, ExternalMap) \
+  V(Map, message_object_map, JSMessageObjectMap)
 
 #define MUTABLE_ROOT_LIST(V)            \
   STRONG_MUTABLE_IMMOVABLE_ROOT_LIST(V) \
@@ -485,7 +541,10 @@ enum class RootIndex : uint16_t {
       kFirstImmortalImmovableRoot + kImmortalImmovableRootsCount - 1,
 
   kFirstSmiRoot = kLastStrongRoot + 1,
-  kLastSmiRoot = kLastRoot
+  kLastSmiRoot = kLastRoot,
+
+  kFirstBuiltinWithSfiRoot = kProxyRevokeSharedFun,
+  kLastBuiltinWithSfiRoot = kFirstBuiltinWithSfiRoot + BUILTINS_WITH_SFI_ROOTS_LIST(COUNT_ROOT) - 1,
 #undef COUNT_ROOT
 };
 // clang-format on
@@ -498,6 +557,10 @@ static_assert(RootIndex::kFirstNameForProtector <=
 NAME_FOR_PROTECTOR_ROOT_LIST(FOR_PROTECTOR_CHECK)
 #undef FOR_PROTECTOR_CHECK
 
+#define ROOT_TYPE_FWD_DECL(Type, name, CamelName) class Type;
+ROOT_LIST(ROOT_TYPE_FWD_DECL)
+#undef ROOT_TYPE_FWD_DECL
+
 // Represents a storage of V8 heap roots.
 class RootsTable {
  public:
@@ -510,7 +573,18 @@ class RootsTable {
                                    RootIndex* index) const;
 
   template <typename T>
-  bool IsRootHandle(Handle<T> handle, RootIndex* index) const;
+  bool IsRootHandle(IndirectHandle<T> handle, RootIndex* index) const;
+
+  // Returns heap number with identical value if it already exists or the empty
+  // handle otherwise.
+  IndirectHandle<HeapNumber> FindHeapNumber(double value);
+
+#define ROOT_ACCESSOR(Type, name, CamelName) \
+  V8_INLINE IndirectHandle<Type> name();
+  ROOT_LIST(ROOT_ACCESSOR)
+#undef ROOT_ACCESSOR
+
+  V8_INLINE IndirectHandle<Object> handle_at(RootIndex root_index);
 
   Address const& operator[](RootIndex root_index) const {
     size_t index = static_cast<size_t>(root_index);
@@ -553,6 +627,15 @@ class RootsTable {
            static_cast<unsigned>(RootIndex::kLastReadOnlyRoot);
   }
 
+  static constexpr RootIndex SingleCharacterStringIndex(int c) {
+    DCHECK_GE(c, 0);
+    DCHECK_LE(c, static_cast<unsigned>(RootIndex::klatin1_ff_string) -
+                     static_cast<unsigned>(RootIndex::kascii_nul_string));
+    static_assert(static_cast<int>(RootIndex::kFirstReadOnlyRoot) == 0);
+    return static_cast<RootIndex>(
+        static_cast<unsigned>(RootIndex::kascii_nul_string) + c);
+  }
+
  private:
   FullObjectSlot begin() {
     return FullObjectSlot(&roots_[static_cast<size_t>(RootIndex::kFirstRoot)]);
@@ -617,16 +700,12 @@ class RootsTable {
   friend class Factory;
   friend class FactoryBase<Factory>;
   friend class FactoryBase<LocalFactory>;
-  friend class PointerCompressedReadOnlyArtifacts;
   friend class ReadOnlyHeap;
   friend class ReadOnlyRoots;
   friend class RootsSerializer;
-  friend class SoleReadOnlyHeap;
 };
 
-#define ROOT_TYPE_FWD_DECL(Type, name, CamelName) class Type;
-READ_ONLY_ROOT_LIST(ROOT_TYPE_FWD_DECL)
-#undef ROOT_TYPE_FWD_DECL
+inline ReadOnlyRoots GetReadOnlyRoots();
 
 class ReadOnlyRoots {
  public:
@@ -641,10 +720,9 @@ class ReadOnlyRoots {
   // map-word instead of a tagged heap pointer.
   MapWord one_pointer_filler_map_word();
 
-#define ROOT_ACCESSOR(Type, name, CamelName)       \
-  V8_INLINE Tagged<Type> name() const;             \
-  V8_INLINE Tagged<Type> unchecked_##name() const; \
-  V8_INLINE Handle<Type> name##_handle() const;
+#define ROOT_ACCESSOR(Type, name, CamelName) \
+  V8_INLINE Tagged<Type> name() const;       \
+  V8_INLINE Tagged<Type> unchecked_##name() const;
 
   READ_ONLY_ROOT_LIST(ROOT_ACCESSOR)
 #undef ROOT_ACCESSOR
@@ -653,20 +731,15 @@ class ReadOnlyRoots {
   V8_INLINE void VerifyNameForProtectorsPages() const;
 #ifdef DEBUG
   void VerifyNameForProtectors();
+  void VerifyTypes();
 #endif
 
   V8_INLINE Tagged<Boolean> boolean_value(bool value) const;
-  V8_INLINE Handle<Boolean> boolean_value_handle(bool value) const;
-
-  // Returns heap number with identical value if it already exists or the empty
-  // handle otherwise.
-  Handle<HeapNumber> FindHeapNumber(double value);
 
   V8_INLINE Address address_at(RootIndex root_index) const;
   V8_INLINE Tagged<Object> object_at(RootIndex root_index) const;
-  V8_INLINE Handle<Object> handle_at(RootIndex root_index) const;
 
-  // Check if a slot is initialized yet. Should only be neccessary for code
+  // Check if a slot is initialized yet. Should only be necessary for code
   // running during snapshot creation.
   V8_INLINE bool is_initialized(RootIndex root_index) const;
 
@@ -682,24 +755,16 @@ class ReadOnlyRoots {
  private:
   V8_INLINE Address first_name_for_protector() const;
   V8_INLINE Address last_name_for_protector() const;
-#ifdef DEBUG
-#define ROOT_TYPE_CHECK(Type, name, CamelName) \
-  V8_EXPORT_PRIVATE bool CheckType_##name() const;
-
-  READ_ONLY_ROOT_LIST(ROOT_TYPE_CHECK)
-#undef ROOT_TYPE_CHECK
-#endif
 
   V8_INLINE explicit ReadOnlyRoots(Address* ro_roots)
       : read_only_roots_(ro_roots) {}
 
-  V8_INLINE Address* GetLocation(RootIndex root_index) const;
-
   Address* read_only_roots_;
 
   friend class ReadOnlyHeap;
   friend class DeserializerAllocator;
   friend class ReadOnlyHeapImageDeserializer;
+  friend ReadOnlyRoots GetReadOnlyRoots();
 };
 
 }  // namespace internal
