@@ -648,7 +648,8 @@ function normalizeSpawnArguments(file, args, options) {
         'DEP0190');
       emittedDEP0190Already = true;
     }
-    const command = ArrayPrototypeJoin([file, ...args], ' ');
+
+    const command = args.length > 0 ? `${file} ${ArrayPrototypeJoin(args, ' ')}` : file;
     // Set the shell, switches, and commands.
     if (process.platform === 'win32') {
       if (typeof options.shell === 'string')
