@@ -12,6 +12,8 @@ const {
   Boolean,
   ErrorCaptureStackTrace,
   FunctionPrototypeBind,
+  MapPrototypeGet,
+  MapPrototypeValues,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectKeys,
@@ -139,12 +141,20 @@ function Console(options /* or: stdout, stderr, ignoreErrors = true */) {
   if (inspectOptions !== undefined) {
     validateObject(inspectOptions, 'options.inspectOptions');
 
-    if (inspectOptions.colors !== undefined &&
-        options.colorMode !== undefined) {
-      throw new ERR_INCOMPATIBLE_OPTION_PAIR(
-        'options.inspectOptions.color', 'colorMode');
+    const inspectOptionsMap = isMap(inspectOptions) ?
+      inspectOptions : new SafeMap([
+        [stdout, inspectOptions],
+        [stderr, inspectOptions],
+      ]);
+
+    for (const inspectOptions of MapPrototypeValues(inspectOptionsMap)) {
+      if (inspectOptions.colors !== undefined &&
+          options.colorMode !== undefined) {
+        throw new ERR_INCOMPATIBLE_OPTION_PAIR(
+          'options.inspectOptions.color', 'colorMode');
+      }
     }
-    optionsMap.set(this, inspectOptions);
+    optionsMap.set(this, inspectOptionsMap);
   }
 
   // Bind the prototype functions to this Console instance
@@ -316,7 +326,8 @@ ObjectDefineProperties(Console.prototype, {
         color = lazyUtilColors().shouldColorize(stream);
       }
 
-      const options = optionsMap.get(this);
+      const inspectOptionsMap = optionsMap.get(this);
+      const options = inspectOptionsMap ? MapPrototypeGet(inspectOptionsMap, stream) : undefined;
       if (options) {
         if (options.colors === undefined) {
           options.colors = color;
