@@ -10,9 +10,9 @@ const {
   kResourceType,
   getMonotonicTime,
   getNextRequestId,
+  registerDiagnosticChannels,
   sniffMimeType,
 } = require('internal/inspector/network');
-const dc = require('diagnostics_channel');
 const { Network } = require('inspector');
 const { Buffer } = require('buffer');
 
@@ -239,31 +239,14 @@ function onWebSocketClose({ websocket }) {
   });
 }
 
-function enable() {
-  dc.subscribe('undici:request:create', onClientRequestStart);
-  dc.subscribe('undici:request:error', onClientRequestError);
-  dc.subscribe('undici:request:headers', onClientResponseHeaders);
-  dc.subscribe('undici:request:trailers', onClientResponseFinish);
-  dc.subscribe('undici:request:bodyChunkSent', onClientRequestBodyChunkSent);
-  dc.subscribe('undici:request:bodySent', onClientRequestBodySent);
-  dc.subscribe('undici:request:bodyChunkReceived', onClientRequestBodyChunkReceived);
-  dc.subscribe('undici:websocket:open', onWebSocketOpen);
-  dc.subscribe('undici:websocket:close', onWebSocketClose);
-}
-
-function disable() {
-  dc.unsubscribe('undici:request:create', onClientRequestStart);
-  dc.unsubscribe('undici:request:error', onClientRequestError);
-  dc.unsubscribe('undici:request:headers', onClientResponseHeaders);
-  dc.unsubscribe('undici:request:trailers', onClientResponseFinish);
-  dc.unsubscribe('undici:request:bodyChunkSent', onClientRequestBodyChunkSent);
-  dc.unsubscribe('undici:request:bodySent', onClientRequestBodySent);
-  dc.unsubscribe('undici:request:bodyChunkReceived', onClientRequestBodyChunkReceived);
-  dc.unsubscribe('undici:websocket:open', onWebSocketOpen);
-  dc.unsubscribe('undici:websocket:close', onWebSocketClose);
-}
-
-module.exports = {
-  enable,
-  disable,
-};
+module.exports = registerDiagnosticChannels([
+  ['undici:request:create', onClientRequestStart],
+  ['undici:request:error', onClientRequestError],
+  ['undici:request:headers', onClientResponseHeaders],
+  ['undici:request:trailers', onClientResponseFinish],
+  ['undici:request:bodyChunkSent', onClientRequestBodyChunkSent],
+  ['undici:request:bodySent', onClientRequestBodySent],
+  ['undici:request:bodyChunkReceived', onClientRequestBodyChunkReceived],
+  ['undici:websocket:open', onWebSocketOpen],
+  ['undici:websocket:close', onWebSocketClose],
+]);
