@@ -504,7 +504,9 @@ class ModuleLoader {
     const loadResult = this.#loadSync(url, { format, importAttributes });
 
     // Use the synchronous commonjs translator which can deal with cycles.
-    const finalFormat = loadResult.format === 'commonjs' ? 'commonjs-sync' : loadResult.format;
+    const finalFormat =
+      loadResult.format === 'commonjs' ||
+      loadResult.format === 'commonjs-typescript' ? 'commonjs-sync' : loadResult.format;
 
     if (finalFormat === 'wasm') {
       assert.fail('WASM is currently unsupported by require(esm)');
