@@ -22,7 +22,7 @@ const { validateString } = require('internal/validators');
 const fs = require('fs'); // Import all of `fs` so that it can be monkey-patched.
 const internalFS = require('internal/fs/utils');
 const path = require('path');
-const { pathToFileURL, fileURLToPath } = require('internal/url');
+const { pathToFileURL, fileURLToPath, URL } = require('internal/url');
 const assert = require('internal/assert');
 
 const { getOptionValue } = require('internal/options');
@@ -295,13 +295,32 @@ function normalizeReferrerURL(referrerName) {
 }
 
 /**
+ * Coerce a URL string to a filename. This is used by the ESM loader
+ * to map ESM URLs to entries in the CJS module cache on a best-effort basis.
+ * TODO(joyeecheung): this can be rather expensive, cache the result on the
+ * ModuleWrap wherever we can.
  * @param {string|undefined} url URL to convert to filename
  * @returns {string|undefined}
  */
 function urlToFilename(url) {
   if (url && StringPrototypeStartsWith(url, 'file://')) {
-    return fileURLToPath(url);
+    let urlObj;
+    try {
+      urlObj = new URL(url);
+    } catch {
+      // Not a proper URL, return as-is as the cache key.
+      return url;
+    }
+    try {
+      return fileURLToPath(urlObj);
+    } catch {
+      // This is generally only possible when the URL is provided by a custom loader.
+      // Just use the path and ignore whether it's absolute or not as there's no such
+      // requirement for CJS cache.
+      return urlObj.pathname;
+    }
   }
+  // Not a file URL, return as-is.
   return url;
 }
 
