@@ -35,6 +35,11 @@ const {
   moveCursor,
 } = require('internal/readline/callbacks');
 
+const {
+  kIsMultiline,
+  kSetLine,
+} = require('internal/readline/interface');
+
 const {
   commonPrefix,
   kSubstringSearch,
@@ -292,7 +297,7 @@ function setupPreview(repl, contextSymbol, bufferSymbol, active) {
   function getInputPreview(input, callback) {
     // For similar reasons as `defaultEval`, wrap expressions starting with a
     // curly brace with parenthesis.
-    if (!wrapped && input[0] === '{' && input[input.length - 1] !== ';') {
+    if (!wrapped && input[0] === '{' && input[input.length - 1] !== ';' && isValidSyntax(input)) {
       input = `(${input})`;
       wrapped = true;
     }
@@ -357,8 +362,11 @@ function setupPreview(repl, contextSymbol, bufferSymbol, active) {
   }
 
   const showPreview = (showCompletion = true) => {
-    // Prevent duplicated previews after a refresh.
-    if (inputPreview !== null || !repl.isCompletionEnabled || !process.features.inspector) {
+    // Prevent duplicated previews after a refresh or in a multiline command.
+    if (inputPreview !== null ||
+        repl[kIsMultiline] ||
+        !repl.isCompletionEnabled ||
+        !process.features.inspector) {
       return;
     }
 
@@ -440,9 +448,14 @@ function setupPreview(repl, contextSymbol, bufferSymbol, active) {
 
       const { cursorPos, displayPos } = getPreviewPos();
       const rows = displayPos.rows - cursorPos.rows;
+      // Moves one line below all the user lines
       moveCursor(repl.output, 0, rows);
+      // Writes the preview there
       repl.output.write(`\n${result}`);
+
+      // Go back to the horizontal position of the cursor
       cursorTo(repl.output, cursorPos.cols);
+      // Go back to the vertical position of the cursor
       moveCursor(repl.output, 0, -rows - 1);
     };
 
@@ -668,7 +681,7 @@ function setupReverseSearch(repl) {
     // line to the found entry.
     if (!isInReverseSearch) {
       if (lastMatch !== -1) {
-        repl.line = repl.history[lastMatch];
+        repl[kSetLine](repl.history[lastMatch]);
         repl.cursor = lastCursor;
         repl.historyIndex = lastMatch;
       }
@@ -737,6 +750,25 @@ function setupReverseSearch(repl) {
 
 const startsWithBraceRegExp = /^\s*{/;
 const endsWithSemicolonRegExp = /;\s*$/;
+function isValidSyntax(input) {
+  try {
+    AcornParser.parse(input, {
+      ecmaVersion: 'latest',
+      allowAwaitOutsideFunction: true,
+    });
+    return true;
+  } catch {
+    try {
+      AcornParser.parse(`_=${input}`, {
+        ecmaVersion: 'latest',
+        allowAwaitOutsideFunction: true,
+      });
+      return true;
+    } catch {
+      return false;
+    }
+  }
+}
 
 /**
  * Checks if some provided code represents an object literal.
@@ -759,4 +791,5 @@ module.exports = {
   setupPreview,
   setupReverseSearch,
   isObjectLiteral,
+  isValidSyntax,
 };
