@@ -35,7 +35,7 @@ const { spawn } = require('child_process');
 const { finished } = require('internal/streams/end-of-stream');
 const { resolve, sep, isAbsolute } = require('path');
 const { DefaultDeserializer, DefaultSerializer } = require('v8');
-const { getOptionValue } = require('internal/options');
+const { getOptionValue, getOptionsAsFlagsFromBinding } = require('internal/options');
 const { Interface } = require('internal/readline/interface');
 const { deserializeError } = require('internal/error_serdes');
 const { Buffer } = require('buffer');
@@ -103,9 +103,12 @@ const kFilterArgs = [
   '--experimental-test-coverage',
   '--watch',
   '--experimental-default-config-file',
+];
+const kFilterArgValues = [
+  '--test-reporter',
+  '--test-reporter-destination',
   '--experimental-config-file',
 ];
-const kFilterArgValues = ['--test-reporter', '--test-reporter-destination'];
 const kDiagnosticsFilterArgs = ['tests', 'suites', 'pass', 'fail', 'cancelled', 'skipped', 'todo', 'duration_ms'];
 
 const kCanceledTests = new SafeSet()
@@ -145,36 +148,45 @@ function getRunArgs(path, { forceExit,
                             only,
                             argv: suppliedArgs,
                             execArgv,
+                            rerunFailuresFilePath,
+                            root: { timeout },
                             cwd }) {
-  const argv = ArrayPrototypeFilter(process.execArgv, filterExecArgv);
+  const processNodeOptions = getOptionsAsFlagsFromBinding();
+  const runArgs = ArrayPrototypeFilter(processNodeOptions, filterExecArgv);
   if (forceExit === true) {
-    ArrayPrototypePush(argv, '--test-force-exit');
+    ArrayPrototypePush(runArgs, '--test-force-exit');
   }
   if (isUsingInspector()) {
-    ArrayPrototypePush(argv, `--inspect-port=${getInspectPort(inspectPort)}`);
+    ArrayPrototypePush(runArgs, `--inspect-port=${getInspectPort(inspectPort)}`);
   }
   if (testNamePatterns != null) {
-    ArrayPrototypeForEach(testNamePatterns, (pattern) => ArrayPrototypePush(argv, `--test-name-pattern=${pattern}`));
+    ArrayPrototypeForEach(testNamePatterns, (pattern) => ArrayPrototypePush(runArgs, `--test-name-pattern=${pattern}`));
   }
   if (testSkipPatterns != null) {
-    ArrayPrototypeForEach(testSkipPatterns, (pattern) => ArrayPrototypePush(argv, `--test-skip-pattern=${pattern}`));
+    ArrayPrototypeForEach(testSkipPatterns, (pattern) => ArrayPrototypePush(runArgs, `--test-skip-pattern=${pattern}`));
   }
   if (only === true) {
-    ArrayPrototypePush(argv, '--test-only');
+    ArrayPrototypePush(runArgs, '--test-only');
+  }
+  if (timeout != null) {
+    ArrayPrototypePush(runArgs, `--test-timeout=${timeout}`);
+  }
+  if (rerunFailuresFilePath) {
+    ArrayPrototypePush(runArgs, `--test-rerun-failures=${rerunFailuresFilePath}`);
   }
 
-  ArrayPrototypePushApply(argv, execArgv);
+  ArrayPrototypePushApply(runArgs, execArgv);
 
   if (path === kIsolatedProcessName) {
-    ArrayPrototypePush(argv, '--test');
-    ArrayPrototypePushApply(argv, ArrayPrototypeSlice(process.argv, 1));
+    ArrayPrototypePush(runArgs, '--test');
+    ArrayPrototypePushApply(runArgs, ArrayPrototypeSlice(process.argv, 1));
   } else {
-    ArrayPrototypePush(argv, path);
+    ArrayPrototypePush(runArgs, path);
   }
 
-  ArrayPrototypePushApply(argv, suppliedArgs);
+  ArrayPrototypePushApply(runArgs, suppliedArgs);
 
-  return argv;
+  return runArgs;
 }
 
 const serializer = new DefaultSerializer();
@@ -580,6 +592,7 @@ function run(options = kEmptyObject) {
     execArgv = [],
     argv = [],
     cwd = process.cwd(),
+    rerunFailuresFilePath,
   } = options;
 
   if (files != null) {
@@ -612,6 +625,10 @@ function run(options = kEmptyObject) {
     );
   }
 
+  if (rerunFailuresFilePath) {
+    validatePath(rerunFailuresFilePath, 'options.rerunFailuresFilePath');
+  }
+
   if (shard != null) {
     validateObject(shard, 'options.shard');
     // Avoid re-evaluating the shard object in case it's a getter
@@ -694,6 +711,7 @@ function run(options = kEmptyObject) {
     coverage,
     coverageExcludeGlobs,
     coverageIncludeGlobs,
+    rerunFailuresFilePath,
     lineCoverage: lineCoverage,
     branchCoverage: branchCoverage,
     functionCoverage: functionCoverage,
@@ -727,6 +745,7 @@ function run(options = kEmptyObject) {
     isolation,
     argv,
     execArgv,
+    rerunFailuresFilePath,
   };
 
   if (isolation === 'process') {
