@@ -24,7 +24,10 @@
 
 const {
   ArrayPrototypeSome,
+  ObjectEntries,
+  ObjectPrototypeHasOwnProperty: hasOwn,
   RegExpPrototypeExec,
+  SafeMap,
   StringPrototypeSplit,
   StringPrototypeToLowerCase,
 } = primordials;
@@ -64,17 +67,31 @@ const TERM_ENVS = {
   'rxvt-unicode-24bit': COLORS_16m,
   // https://bugs.launchpad.net/terminator/+bug/1030562
   'terminator': COLORS_16m,
+  'xterm-kitty': COLORS_16m,
 };
 
+const CI_ENVS_MAP = new SafeMap(ObjectEntries({
+  APPVEYOR: COLORS_256,
+  BUILDKITE: COLORS_256,
+  CIRCLECI: COLORS_16m,
+  DRONE: COLORS_256,
+  GITEA_ACTIONS: COLORS_16m,
+  GITHUB_ACTIONS: COLORS_16m,
+  GITLAB_CI: COLORS_256,
+  TRAVIS: COLORS_256,
+}));
+
 const TERM_ENVS_REG_EXP = [
   /ansi/,
   /color/,
   /linux/,
+  /direct/,
   /^con[0-9]*x[0-9]/,
   /^rxvt/,
   /^screen/,
   /^xterm/,
   /^vt100/,
+  /^vt220/,
 ];
 
 let warned = false;
@@ -82,9 +99,9 @@ function warnOnDeactivatedColors(env) {
   if (warned)
     return;
   let name = '';
-  if (env.NODE_DISABLE_COLORS !== undefined)
+  if (env.NODE_DISABLE_COLORS !== undefined && env.NODE_DISABLE_COLORS !== '')
     name = 'NODE_DISABLE_COLORS';
-  if (env.NO_COLOR !== undefined) {
+  if (env.NO_COLOR !== undefined && env.NO_COLOR !== '') {
     if (name !== '') {
       name += "' and '";
     }
@@ -124,9 +141,9 @@ function getColorDepth(env = process.env) {
     }
   }
 
-  if (env.NODE_DISABLE_COLORS !== undefined ||
+  if ((env.NODE_DISABLE_COLORS !== undefined && env.NODE_DISABLE_COLORS !== '') ||
       // See https://no-color.org/
-      env.NO_COLOR !== undefined ||
+      (env.NO_COLOR !== undefined && env.NO_COLOR !== '') ||
       // The "dumb" special terminal, as defined by terminfo, doesn't support
       // ANSI color control codes.
       // See https://invisible-island.net/ncurses/terminfo.ti.html#toc-_Specials
@@ -155,19 +172,21 @@ function getColorDepth(env = process.env) {
   }
 
   if (env.TMUX) {
-    return COLORS_256;
+    return COLORS_16m;
   }
 
-  if (env.CI) {
-    if ([
-      'APPVEYOR',
-      'BUILDKITE',
-      'CIRCLECI',
-      'DRONE',
-      'GITHUB_ACTIONS',
-      'GITLAB_CI',
-      'TRAVIS',
-    ].some((sign) => sign in env) || env.CI_NAME === 'codeship') {
+  // Azure DevOps
+  if (hasOwn(env, 'TF_BUILD') && hasOwn(env, 'AGENT_NAME')) {
+    return COLORS_16;
+  }
+
+  if (hasOwn(env, 'CI')) {
+    for (const { 0: envName, 1: colors } of CI_ENVS_MAP) {
+      if (hasOwn(env, envName)) {
+        return colors;
+      }
+    }
+    if (env.CI_NAME === 'codeship') {
       return COLORS_256;
     }
     return COLORS_2;
@@ -198,6 +217,10 @@ function getColorDepth(env = process.env) {
   }
 
   if (env.TERM) {
+    if (RegExpPrototypeExec(/truecolor/, env.TERM) !== null) {
+      return COLORS_16m;
+    }
+
     if (RegExpPrototypeExec(/^xterm-256/, env.TERM) !== null) {
       return COLORS_256;
     }
