@@ -1,5 +1,3 @@
-// Flags: --js-float16array
-// TODO(LiviaMedeiros): once `Float16Array` is unflagged in v8, remove the line above
 'use strict';
 
 const common = require('../common');
@@ -7,9 +5,6 @@ const vm = require('node:vm');
 const assert = require('node:assert');
 const { describe, it } = require('node:test');
 
-// TODO(LiviaMedeiros): once linter recognizes `Float16Array`, remove next line
-const { Float16Array } = globalThis;
-
 const x = ['x'];
 
 function createCircularObject() {
