@@ -3,20 +3,25 @@
 require('../common');
 const assert = require('assert');
 const repl = require('repl');
-const stream = require('stream');
+const { startNewREPLServer } = require('../common/repl');
+
+const testingReplPrompt = '_REPL_TESTING_PROMPT_>';
 
 testSloppyMode();
 testStrictMode();
+testMagicMode();
 testResetContext();
 testResetContextGlobal();
-testMagicMode();
 testError();
 
 function testSloppyMode() {
-  const r = initRepl(repl.REPL_MODE_SLOPPY);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    mode: repl.REPL_MODE_SLOPPY,
+  });
 
   // Cannot use `let` in sloppy mode
-  r.write(`_;          // initial value undefined
+  replServer.write(`_;          // initial value undefined
           var x = 10;  // evaluates to undefined
           _;           // still undefined
           y = 10;      // evaluates to 10
@@ -29,7 +34,7 @@ function testSloppyMode() {
           _;           // remains 30 from user input
           `);
 
-  assertOutput(r.output, [
+  assertOutput(output, [
     'undefined',
     'undefined',
     'undefined',
@@ -46,9 +51,12 @@ function testSloppyMode() {
 }
 
 function testStrictMode() {
-  const r = initRepl(repl.REPL_MODE_STRICT);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    mode: repl.REPL_MODE_STRICT,
+  });
 
-  r.write(`_;          // initial value undefined
+  replServer.write(`_;          // initial value undefined
           var x = 10;  // evaluates to undefined
           _;           // still undefined
           let _ = 20;  // use 'let' only in strict mode - evals to undefined
@@ -62,7 +70,7 @@ function testStrictMode() {
           _;           // remains 30 from user input
           `);
 
-  assertOutput(r.output, [
+  assertOutput(output, [
     'undefined',
     'undefined',
     'undefined',
@@ -79,9 +87,12 @@ function testStrictMode() {
 }
 
 function testMagicMode() {
-  const r = initRepl(repl.REPL_MODE_MAGIC);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    mode: repl.REPL_MODE_MAGIC,
+  });
 
-  r.write(`_;          // initial value undefined
+  replServer.write(`_;          // initial value undefined
           x = 10;      //
           _;           // last eval - 10
           let _ = 20;  // undefined
@@ -95,7 +106,7 @@ function testMagicMode() {
           _;           // remains 30 from user input
           `);
 
-  assertOutput(r.output, [
+  assertOutput(output, [
     'undefined',
     '10',
     '10',
@@ -112,9 +123,12 @@ function testMagicMode() {
 }
 
 function testResetContext() {
-  const r = initRepl(repl.REPL_MODE_SLOPPY);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    mode: repl.REPL_MODE_MAGIC,
+  });
 
-  r.write(`_ = 10;     // explicitly set to 10
+  replServer.write(`_ = 10;     // explicitly set to 10
           _;           // 10 from user input
           .clear       // Clearing context...
           _;           // remains 10
@@ -122,7 +136,7 @@ function testResetContext() {
           _;           // expect 20
           `);
 
-  assertOutput(r.output, [
+  assertOutput(output, [
     'Expression assignment to _ now disabled.',
     '10',
     '10',
@@ -134,15 +148,18 @@ function testResetContext() {
 }
 
 function testResetContextGlobal() {
-  const r = initRepl(repl.REPL_MODE_STRICT, true);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    useGlobal: true,
+  });
 
-  r.write(`_ = 10;     // explicitly set to 10
+  replServer.write(`_ = 10;     // explicitly set to 10
           _;           // 10 from user input
           .clear       // No output because useGlobal is true
           _;           // remains 10
           `);
 
-  assertOutput(r.output, [
+  assertOutput(output, [
     'Expression assignment to _ now disabled.',
     '10',
     '10',
@@ -155,9 +172,15 @@ function testResetContextGlobal() {
 }
 
 function testError() {
-  const r = initRepl(repl.REPL_MODE_STRICT);
+  const { replServer, output } = startNewREPLServer({
+    prompt: testingReplPrompt,
+    replMode: repl.REPL_MODE_STRICT,
+    preview: false,
+  }, {
+    disableDomainErrorAssert: true
+  });
 
-  r.write(`_error;                                // initial value undefined
+  replServer.write(`_error;                                // initial value undefined
            throw new Error('foo');                // throws error
            _error;                                // shows error
            fs.readdirSync('/nonexistent?');       // throws error, sync
@@ -168,7 +191,9 @@ function testError() {
            `);
 
   setImmediate(() => {
-    const lines = r.output.accum.trim().split('\n');
+    const lines = output.accumulator.trim().split('\n').filter(
+      (line) => !line.includes(testingReplPrompt) || line.includes('Uncaught Error')
+    );
     const expectedLines = [
       'undefined',
 
@@ -192,7 +217,7 @@ function testError() {
       'undefined',
 
       // The message from the original throw
-      'Uncaught Error: baz',
+      /Uncaught Error: baz/,
     ];
     for (const line of lines) {
       const expected = expectedLines.shift();
@@ -204,14 +229,14 @@ function testError() {
     assert.strictEqual(expectedLines.length, 0);
 
     // Reset output, check that '_error' is the asynchronously caught error.
-    r.output.accum = '';
-    r.write(`_error.message                 // show the message
+    output.accumulator = '';
+    replServer.write(`_error.message                 // show the message
              _error = 0;                    // disable auto-assignment
              throw new Error('quux');       // new error
              _error;                        // should not see the new error
              `);
 
-    assertOutput(r.output, [
+    assertOutput(output, [
       "'baz'",
       'Expression assignment to _error now disabled.',
       '0',
@@ -221,27 +246,7 @@ function testError() {
   });
 }
 
-function initRepl(mode, useGlobal) {
-  const inputStream = new stream.PassThrough();
-  const outputStream = new stream.PassThrough();
-  outputStream.accum = '';
-
-  outputStream.on('data', (data) => {
-    outputStream.accum += data;
-  });
-
-  return repl.start({
-    input: inputStream,
-    output: outputStream,
-    useColors: false,
-    terminal: false,
-    prompt: '',
-    replMode: mode,
-    useGlobal: useGlobal
-  });
-}
-
 function assertOutput(output, expected) {
-  const lines = output.accum.trim().split('\n');
+  const lines = output.accumulator.trim().split('\n').filter((line) => !line.includes(testingReplPrompt));
   assert.deepStrictEqual(lines, expected);
 }
